"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Thrown when there is a problem with a given API call. This is special-cased in the responder
 * to create a JSON error response with the status code given.
 */
class ApiError {
    /**
     * Creates a new API error
     * @param {number} statusCode The HTTP status code to return
     * @param {string|object} json An object to be returned as JSON or a message to be returned (which is
     * then converted to JSON as {message: "your_message"})
     * @param {string} errCode The internal error code to describe what went wrong
     */
    constructor(statusCode, json, errCode = "M_UNKNOWN") {
        // Because typescript is just plain dumb
        // https://stackoverflow.com/questions/31626231/custom-error-class-in-typescript
        Error.apply(this, ["ApiError"]);
        if (typeof (json) === "string")
            json = { message: json };
        this.jsonResponse = json;
        this.statusCode = statusCode;
        this.errorCode = errCode;
        this.jsonResponse["dim_errcode"] = this.errorCode;
        if (!this.jsonResponse['error'])
            this.jsonResponse['error'] = this.jsonResponse['message'];
        if (!this.jsonResponse['errcode'])
            this.jsonResponse['errcode'] = errCode;
    }
}
exports.ApiError = ApiError;
//# sourceMappingURL=ApiError.js.map