"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const typescript_rest_1 = require("typescript-rest");
const AppService_1 = require("../../db/models/AppService");
const AppserviceStore_1 = require("../../db/AppserviceStore");
const ApiError_1 = require("../ApiError");
const MatrixAppserviceClient_1 = require("../../matrix/MatrixAppserviceClient");
const matrix_js_snippets_1 = require("matrix-js-snippets");
const MatrixSecurity_1 = require("../security/MatrixSecurity");
/**
 * Administrative API for managing the appservices that Dimension operates.
 */
let AdminAppserviceService = class AdminAppserviceService {
    getAppservices() {
        return __awaiter(this, void 0, void 0, function* () {
            return (yield AppService_1.default.findAll()).map(a => this.mapAppservice(a));
        });
    }
    getAppservice(asId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const appservice = yield AppserviceStore_1.AppserviceStore.getAppservice(asId);
                return this.mapAppservice(appservice);
            }
            catch (err) {
                matrix_js_snippets_1.LogService.error("AdminAppserviceService", err);
                throw new ApiError_1.ApiError(404, "Appservice not found");
            }
        });
    }
    createAppservice(request) {
        return __awaiter(this, void 0, void 0, function* () {
            const userId = this.context.request.user.userId;
            // Trim off the @ sign if it's on the prefix
            if (request.userPrefix[0] === "@") {
                request.userPrefix = request.userPrefix.substring(1);
            }
            const appservices = yield AppserviceStore_1.AppserviceStore.getAllByUserPrefix(request.userPrefix);
            if (appservices && appservices.length > 0) {
                throw new ApiError_1.ApiError(400, "User prefix is already in use");
            }
            const appservice = yield AppserviceStore_1.AppserviceStore.create(AppserviceStore_1.AppserviceStore.getSafeUserId(request.userPrefix));
            matrix_js_snippets_1.LogService.info("AdminAppserviceService", userId + " created an application service");
            return this.mapAppservice(appservice);
        });
    }
    test(asId) {
        return __awaiter(this, void 0, void 0, function* () {
            const appservice = yield AppserviceStore_1.AppserviceStore.getAppservice(asId);
            const client = new MatrixAppserviceClient_1.MatrixAppserviceClient(appservice);
            const userId = yield client.whoAmI();
            if (userId.startsWith("@" + appservice.userPrefix))
                return {}; // 200 OK
            throw new ApiError_1.ApiError(500, "User ID does not match the application service");
        });
    }
    mapAppservice(as) {
        return {
            id: as.id,
            hsToken: as.hsToken,
            asToken: as.asToken,
            userPrefix: as.userPrefix,
        };
    }
};
__decorate([
    typescript_rest_1.Context,
    __metadata("design:type", typescript_rest_1.ServiceContext)
], AdminAppserviceService.prototype, "context", void 0);
__decorate([
    typescript_rest_1.GET,
    typescript_rest_1.Path("all"),
    typescript_rest_1.Security([MatrixSecurity_1.ROLE_USER, MatrixSecurity_1.ROLE_ADMIN]),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], AdminAppserviceService.prototype, "getAppservices", null);
__decorate([
    typescript_rest_1.GET,
    typescript_rest_1.Path(":appserviceId"),
    typescript_rest_1.Security([MatrixSecurity_1.ROLE_USER, MatrixSecurity_1.ROLE_ADMIN]),
    __param(0, typescript_rest_1.PathParam("appserviceId")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", Promise)
], AdminAppserviceService.prototype, "getAppservice", null);
__decorate([
    typescript_rest_1.POST,
    typescript_rest_1.Path("new"),
    typescript_rest_1.Security([MatrixSecurity_1.ROLE_USER, MatrixSecurity_1.ROLE_ADMIN]),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], AdminAppserviceService.prototype, "createAppservice", null);
__decorate([
    typescript_rest_1.POST,
    typescript_rest_1.Path(":appserviceId/test"),
    typescript_rest_1.Security([MatrixSecurity_1.ROLE_USER, MatrixSecurity_1.ROLE_ADMIN]),
    __param(0, typescript_rest_1.PathParam("appserviceId")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", Promise)
], AdminAppserviceService.prototype, "test", null);
AdminAppserviceService = __decorate([
    typescript_rest_1.Path("/api/v1/dimension/admin/appservices")
], AdminAppserviceService);
exports.AdminAppserviceService = AdminAppserviceService;
//# sourceMappingURL=AdminAppserviceService.js.map