"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const typescript_rest_1 = require("typescript-rest");
const ApiError_1 = require("../ApiError");
const matrix_js_snippets_1 = require("matrix-js-snippets");
const BotStore_1 = require("../../db/BotStore");
const MemoryCache_1 = require("../../MemoryCache");
const MatrixSecurity_1 = require("../security/MatrixSecurity");
/**
 * Administrative API for managing custom simple bots hosted by Dimension
 */
let AdminCustomSimpleBotService = class AdminCustomSimpleBotService {
    getBots() {
        return __awaiter(this, void 0, void 0, function* () {
            return BotStore_1.BotStore.getCustomBots();
        });
    }
    getBot(botId) {
        return __awaiter(this, void 0, void 0, function* () {
            const bot = yield BotStore_1.BotStore.getCustomBot(botId);
            if (!bot)
                throw new ApiError_1.ApiError(404, "Bot not found");
            return bot;
        });
    }
    createBot(request) {
        return __awaiter(this, void 0, void 0, function* () {
            const userId = this.context.request.user.userId;
            const bot = yield BotStore_1.BotStore.createCustom(request);
            matrix_js_snippets_1.LogService.info("AdminCustomSimpleBotService", userId + " created a simple bot");
            MemoryCache_1.Cache.for(MemoryCache_1.CACHE_INTEGRATIONS).clear();
            return bot;
        });
    }
    updateBot(botId, request) {
        return __awaiter(this, void 0, void 0, function* () {
            const userId = this.context.request.user.userId;
            const bot = yield BotStore_1.BotStore.updateCustom(botId, request);
            matrix_js_snippets_1.LogService.info("AdminCustomSimpleBotService", userId + " updated a simple bot");
            MemoryCache_1.Cache.for(MemoryCache_1.CACHE_INTEGRATIONS).clear();
            return bot;
        });
    }
    deleteBot(botId) {
        return __awaiter(this, void 0, void 0, function* () {
            const userId = this.context.request.user.userId;
            yield BotStore_1.BotStore.deleteCustom(botId);
            matrix_js_snippets_1.LogService.info("AdminCustomSimpleBotService", userId + " deleted a simple bot");
            MemoryCache_1.Cache.for(MemoryCache_1.CACHE_INTEGRATIONS).clear();
            return {}; // 200 OK
        });
    }
    getProfile(userId) {
        return __awaiter(this, void 0, void 0, function* () {
            const profile = yield BotStore_1.BotStore.getProfile(userId);
            return { name: profile.displayName, avatarUrl: profile.avatarMxc };
        });
    }
};
__decorate([
    typescript_rest_1.Context,
    __metadata("design:type", typescript_rest_1.ServiceContext)
], AdminCustomSimpleBotService.prototype, "context", void 0);
__decorate([
    typescript_rest_1.GET,
    typescript_rest_1.Path("all"),
    typescript_rest_1.Security([MatrixSecurity_1.ROLE_USER, MatrixSecurity_1.ROLE_ADMIN]),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], AdminCustomSimpleBotService.prototype, "getBots", null);
__decorate([
    typescript_rest_1.GET,
    typescript_rest_1.Path(":botId"),
    typescript_rest_1.Security([MatrixSecurity_1.ROLE_USER, MatrixSecurity_1.ROLE_ADMIN]),
    __param(0, typescript_rest_1.PathParam("botId")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Number]),
    __metadata("design:returntype", Promise)
], AdminCustomSimpleBotService.prototype, "getBot", null);
__decorate([
    typescript_rest_1.POST,
    typescript_rest_1.Path("new"),
    typescript_rest_1.Security([MatrixSecurity_1.ROLE_USER, MatrixSecurity_1.ROLE_ADMIN]),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], AdminCustomSimpleBotService.prototype, "createBot", null);
__decorate([
    typescript_rest_1.POST,
    typescript_rest_1.Path(":botId"),
    typescript_rest_1.Security([MatrixSecurity_1.ROLE_USER, MatrixSecurity_1.ROLE_ADMIN]),
    __param(0, typescript_rest_1.PathParam("botId")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Number, Object]),
    __metadata("design:returntype", Promise)
], AdminCustomSimpleBotService.prototype, "updateBot", null);
__decorate([
    typescript_rest_1.DELETE,
    typescript_rest_1.Path(":botId"),
    typescript_rest_1.Security([MatrixSecurity_1.ROLE_USER, MatrixSecurity_1.ROLE_ADMIN]),
    __param(0, typescript_rest_1.PathParam("botId")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Number]),
    __metadata("design:returntype", Promise)
], AdminCustomSimpleBotService.prototype, "deleteBot", null);
__decorate([
    typescript_rest_1.GET,
    typescript_rest_1.Path("profile/:userId"),
    typescript_rest_1.Security([MatrixSecurity_1.ROLE_USER, MatrixSecurity_1.ROLE_ADMIN]),
    __param(0, typescript_rest_1.PathParam("userId")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", Promise)
], AdminCustomSimpleBotService.prototype, "getProfile", null);
AdminCustomSimpleBotService = __decorate([
    typescript_rest_1.Path("/api/v1/dimension/admin/bots/simple/custom")
], AdminCustomSimpleBotService);
exports.AdminCustomSimpleBotService = AdminCustomSimpleBotService;
//# sourceMappingURL=AdminCustomSimpleBotService.js.map