"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const typescript_rest_1 = require("typescript-rest");
const ApiError_1 = require("../ApiError");
const DimensionIntegrationsService_1 = require("../dimension/DimensionIntegrationsService");
const WidgetStore_1 = require("../../db/WidgetStore");
const MemoryCache_1 = require("../../MemoryCache");
const matrix_js_snippets_1 = require("matrix-js-snippets");
const BridgeStore_1 = require("../../db/BridgeStore");
const MatrixSecurity_1 = require("../security/MatrixSecurity");
/**
 * Administrative API for managing the integrations for Dimension. This is to enable/disable integrations
 * and set basic options. See the NEB APIs for configuring go-neb instances.
 */
let AdminIntegrationsService = class AdminIntegrationsService {
    setOptions(category, type, body) {
        return __awaiter(this, void 0, void 0, function* () {
            const userId = this.context.request.user.userId;
            if (category === "widget")
                yield WidgetStore_1.WidgetStore.setOptions(type, body.options);
            else
                throw new ApiError_1.ApiError(400, "Unrecognized category");
            matrix_js_snippets_1.LogService.info("AdminIntegrationsService", userId + " updated the integration options for " + category + "/" + type);
            MemoryCache_1.Cache.for(MemoryCache_1.CACHE_INTEGRATIONS).clear();
            return {}; // 200 OK
        });
    }
    setEnabled(category, type, body) {
        return __awaiter(this, void 0, void 0, function* () {
            const userId = this.context.request.user.userId;
            if (category === "widget")
                yield WidgetStore_1.WidgetStore.setEnabled(type, body.enabled);
            else if (category === "bridge")
                yield BridgeStore_1.BridgeStore.setEnabled(type, body.enabled);
            else
                throw new ApiError_1.ApiError(400, "Unrecognized category");
            matrix_js_snippets_1.LogService.info("AdminIntegrationsService", userId + " set " + category + "/" + type + " to " + (body.enabled ? "enabled" : "disabled"));
            MemoryCache_1.Cache.for(MemoryCache_1.CACHE_INTEGRATIONS).clear();
            return {}; // 200 OK
        });
    }
    getAllIntegrations(category) {
        return __awaiter(this, void 0, void 0, function* () {
            const userId = this.context.request.user.userId;
            if (category === "widget")
                return yield DimensionIntegrationsService_1.DimensionIntegrationsService.getWidgets(false);
            else if (category === "bridge")
                return yield DimensionIntegrationsService_1.DimensionIntegrationsService.getBridges(false, userId);
            else
                throw new ApiError_1.ApiError(400, "Unrecongized category");
        });
    }
};
__decorate([
    typescript_rest_1.Context,
    __metadata("design:type", typescript_rest_1.ServiceContext)
], AdminIntegrationsService.prototype, "context", void 0);
__decorate([
    typescript_rest_1.POST,
    typescript_rest_1.Path(":category/:type/options"),
    typescript_rest_1.Security([MatrixSecurity_1.ROLE_USER, MatrixSecurity_1.ROLE_ADMIN]),
    __param(0, typescript_rest_1.PathParam("category")), __param(1, typescript_rest_1.PathParam("type")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String, Object]),
    __metadata("design:returntype", Promise)
], AdminIntegrationsService.prototype, "setOptions", null);
__decorate([
    typescript_rest_1.POST,
    typescript_rest_1.Path(":category/:type/enabled"),
    typescript_rest_1.Security([MatrixSecurity_1.ROLE_USER, MatrixSecurity_1.ROLE_ADMIN]),
    __param(0, typescript_rest_1.PathParam("category")), __param(1, typescript_rest_1.PathParam("type")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String, Object]),
    __metadata("design:returntype", Promise)
], AdminIntegrationsService.prototype, "setEnabled", null);
__decorate([
    typescript_rest_1.GET,
    typescript_rest_1.Path(":category/all"),
    typescript_rest_1.Security([MatrixSecurity_1.ROLE_USER, MatrixSecurity_1.ROLE_ADMIN]),
    __param(0, typescript_rest_1.PathParam("category")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", Promise)
], AdminIntegrationsService.prototype, "getAllIntegrations", null);
AdminIntegrationsService = __decorate([
    typescript_rest_1.Path("/api/v1/dimension/admin/integrations")
], AdminIntegrationsService);
exports.AdminIntegrationsService = AdminIntegrationsService;
//# sourceMappingURL=AdminIntegrationsService.js.map