"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const typescript_rest_1 = require("typescript-rest");
const MemoryCache_1 = require("../../MemoryCache");
const matrix_js_snippets_1 = require("matrix-js-snippets");
const ApiError_1 = require("../ApiError");
const IrcBridgeRecord_1 = require("../../db/models/IrcBridgeRecord");
const IrcBridge_1 = require("../../bridges/IrcBridge");
const Upstream_1 = require("../../db/models/Upstream");
const IrcBridgeNetwork_1 = require("../../db/models/IrcBridgeNetwork");
const MatrixSecurity_1 = require("../security/MatrixSecurity");
/**
 * Administrative API for configuring IRC bridge instances.
 */
let AdminIrcService = class AdminIrcService {
    getBridges() {
        return __awaiter(this, void 0, void 0, function* () {
            const userId = this.context.request.user.userId;
            const bridges = yield IrcBridgeRecord_1.default.findAll();
            const client = new IrcBridge_1.IrcBridge(userId);
            return Promise.all(bridges.map((b) => __awaiter(this, void 0, void 0, function* () {
                let networks = null;
                let isOnline = true;
                try {
                    networks = yield client.getNetworks(b);
                }
                catch (e) {
                    matrix_js_snippets_1.LogService.error("AdminIrcService", e);
                    isOnline = false;
                }
                return {
                    id: b.id,
                    upstreamId: b.upstreamId,
                    provisionUrl: b.provisionUrl,
                    isEnabled: b.isEnabled,
                    availableNetworks: networks,
                    isOnline: isOnline,
                };
            })));
        });
    }
    getBridge(bridgeId) {
        return __awaiter(this, void 0, void 0, function* () {
            const userId = this.context.request.user.userId;
            const ircBridge = yield IrcBridgeRecord_1.default.findByPk(bridgeId);
            if (!ircBridge)
                throw new ApiError_1.ApiError(404, "IRC Bridge not found");
            const client = new IrcBridge_1.IrcBridge(userId);
            let networks = null;
            let isOnline = true;
            try {
                networks = yield client.getNetworks(ircBridge);
            }
            catch (e) {
                matrix_js_snippets_1.LogService.error("AdminIrcService", e);
                isOnline = false;
            }
            return {
                id: ircBridge.id,
                upstreamId: ircBridge.upstreamId,
                provisionUrl: ircBridge.provisionUrl,
                isEnabled: ircBridge.isEnabled,
                availableNetworks: networks,
                isOnline: isOnline,
            };
        });
    }
    setNetworkEnabled(bridgeId, networkId, request) {
        return __awaiter(this, void 0, void 0, function* () {
            const userId = this.context.request.user.userId;
            const ircBridge = yield IrcBridgeRecord_1.default.findByPk(bridgeId);
            if (!ircBridge)
                throw new ApiError_1.ApiError(404, "IRC Bridge not found");
            const localNetworkId = IrcBridge_1.IrcBridge.parseNetworkId(networkId).bridgeNetworkId;
            const network = yield IrcBridgeNetwork_1.default.findOne({
                where: {
                    bridgeId: ircBridge.id,
                    bridgeNetworkId: localNetworkId,
                },
            });
            if (!network)
                throw new ApiError_1.ApiError(404, "Network not found");
            network.isEnabled = request.isEnabled;
            yield network.save();
            matrix_js_snippets_1.LogService.info("AdminIrcService", userId + " toggled the network '" + localNetworkId + "' on bridge " + ircBridge.id);
            MemoryCache_1.Cache.for(MemoryCache_1.CACHE_IRC_BRIDGE).clear();
            return {}; // 200 OK
        });
    }
    newConfigForUpstream(request) {
        return __awaiter(this, void 0, void 0, function* () {
            const userId = this.context.request.user.userId;
            const upstream = yield Upstream_1.default.findByPk(request.upstreamId);
            if (!upstream)
                throw new ApiError_1.ApiError(400, "Upstream not found");
            const bridge = yield IrcBridgeRecord_1.default.create({
                upstreamId: request.upstreamId,
                isEnabled: true,
            });
            matrix_js_snippets_1.LogService.info("AdminIrcService", userId + " created a new IRC Bridge from upstream " + request.upstreamId);
            MemoryCache_1.Cache.for(MemoryCache_1.CACHE_IRC_BRIDGE).clear();
            MemoryCache_1.Cache.for(MemoryCache_1.CACHE_INTEGRATIONS).clear();
            return this.getBridge(bridge.id);
        });
    }
    newSelfhosted(request) {
        return __awaiter(this, void 0, void 0, function* () {
            const userId = this.context.request.user.userId;
            const bridge = yield IrcBridgeRecord_1.default.create({
                provisionUrl: request.provisionUrl,
                isEnabled: true,
            });
            matrix_js_snippets_1.LogService.info("AdminIrcService", userId + " created a new IRC Bridge with provisioning URL " + request.provisionUrl);
            MemoryCache_1.Cache.for(MemoryCache_1.CACHE_IRC_BRIDGE).clear();
            MemoryCache_1.Cache.for(MemoryCache_1.CACHE_INTEGRATIONS).clear();
            return this.getBridge(bridge.id);
        });
    }
};
__decorate([
    typescript_rest_1.Context,
    __metadata("design:type", typescript_rest_1.ServiceContext)
], AdminIrcService.prototype, "context", void 0);
__decorate([
    typescript_rest_1.GET,
    typescript_rest_1.Path("all"),
    typescript_rest_1.Security([MatrixSecurity_1.ROLE_USER, MatrixSecurity_1.ROLE_ADMIN]),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], AdminIrcService.prototype, "getBridges", null);
__decorate([
    typescript_rest_1.GET,
    typescript_rest_1.Path(":bridgeId"),
    typescript_rest_1.Security([MatrixSecurity_1.ROLE_USER, MatrixSecurity_1.ROLE_ADMIN]),
    __param(0, typescript_rest_1.PathParam("bridgeId")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Number]),
    __metadata("design:returntype", Promise)
], AdminIrcService.prototype, "getBridge", null);
__decorate([
    typescript_rest_1.POST,
    typescript_rest_1.Path(":bridgeId/network/:networkId/enabled"),
    typescript_rest_1.Security([MatrixSecurity_1.ROLE_USER, MatrixSecurity_1.ROLE_ADMIN]),
    __param(0, typescript_rest_1.PathParam("bridgeId")), __param(1, typescript_rest_1.PathParam("networkId")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Number, String, Object]),
    __metadata("design:returntype", Promise)
], AdminIrcService.prototype, "setNetworkEnabled", null);
__decorate([
    typescript_rest_1.POST,
    typescript_rest_1.Path("new/upstream"),
    typescript_rest_1.Security([MatrixSecurity_1.ROLE_USER, MatrixSecurity_1.ROLE_ADMIN]),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], AdminIrcService.prototype, "newConfigForUpstream", null);
__decorate([
    typescript_rest_1.POST,
    typescript_rest_1.Path("new/selfhosted"),
    typescript_rest_1.Security([MatrixSecurity_1.ROLE_USER, MatrixSecurity_1.ROLE_ADMIN]),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], AdminIrcService.prototype, "newSelfhosted", null);
AdminIrcService = __decorate([
    typescript_rest_1.Path("/api/v1/dimension/admin/irc")
], AdminIrcService);
exports.AdminIrcService = AdminIrcService;
//# sourceMappingURL=AdminIrcService.js.map