"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const typescript_rest_1 = require("typescript-rest");
const MemoryCache_1 = require("../../MemoryCache");
const NebStore_1 = require("../../db/NebStore");
const matrix_js_snippets_1 = require("matrix-js-snippets");
const ApiError_1 = require("../ApiError");
const MatrixSecurity_1 = require("../security/MatrixSecurity");
/**
 * Administrative API for configuring go-neb instances.
 */
let AdminNebService = class AdminNebService {
    getNebConfigs() {
        return __awaiter(this, void 0, void 0, function* () {
            const cachedConfigs = MemoryCache_1.Cache.for(MemoryCache_1.CACHE_NEB).get("configurations");
            if (cachedConfigs)
                return cachedConfigs;
            const configs = yield NebStore_1.NebStore.getAllConfigs();
            MemoryCache_1.Cache.for(MemoryCache_1.CACHE_NEB).put("configurations", configs);
            return configs;
        });
    }
    getNebConfig(nebId) {
        return __awaiter(this, void 0, void 0, function* () {
            const configs = yield this.getNebConfigs();
            const firstConfig = configs.filter(c => c.id === nebId)[0];
            if (!firstConfig)
                throw new ApiError_1.ApiError(404, "Configuration not found");
            return firstConfig;
        });
    }
    setIntegrationEnabled(nebId, integrationType, request) {
        return __awaiter(this, void 0, void 0, function* () {
            const userId = this.context.request.user.userId;
            yield NebStore_1.NebStore.setIntegrationEnabled(nebId, integrationType, request.enabled);
            matrix_js_snippets_1.LogService.info("AdminNebService", userId + " set the " + integrationType + " on NEB " + nebId + " to " + (request.enabled ? "enabled" : "disabled"));
            MemoryCache_1.Cache.for(MemoryCache_1.CACHE_NEB).clear();
            MemoryCache_1.Cache.for(MemoryCache_1.CACHE_INTEGRATIONS).clear();
            return {}; // 200 OK
        });
    }
    setIntegrationConfig(nebId, integrationType, newConfig) {
        return __awaiter(this, void 0, void 0, function* () {
            const userId = this.context.request.user.userId;
            yield NebStore_1.NebStore.setIntegrationConfig(nebId, integrationType, newConfig);
            matrix_js_snippets_1.LogService.info("AdminNebService", userId + " updated the configuration for " + integrationType + " on NEB " + nebId);
            MemoryCache_1.Cache.for(MemoryCache_1.CACHE_NEB).clear();
            MemoryCache_1.Cache.for(MemoryCache_1.CACHE_INTEGRATIONS).clear();
            return {}; // 200 OK
        });
    }
    getIntegrationConfig(nebId, integrationType) {
        return __awaiter(this, void 0, void 0, function* () {
            return NebStore_1.NebStore.getIntegrationConfig(nebId, integrationType);
        });
    }
    newConfigForUpstream(request) {
        return __awaiter(this, void 0, void 0, function* () {
            const userId = this.context.request.user.userId;
            try {
                const neb = yield NebStore_1.NebStore.createForUpstream(request.upstreamId);
                matrix_js_snippets_1.LogService.info("AdminNebService", userId + " created a new NEB instance from upstream " + request.upstreamId);
                MemoryCache_1.Cache.for(MemoryCache_1.CACHE_NEB).clear();
                MemoryCache_1.Cache.for(MemoryCache_1.CACHE_INTEGRATIONS).clear();
                return neb;
            }
            catch (err) {
                matrix_js_snippets_1.LogService.error("DimensionNebAdminService", err);
                throw new ApiError_1.ApiError(500, "Error creating go-neb instance");
            }
        });
    }
    newConfigForAppservice(request) {
        return __awaiter(this, void 0, void 0, function* () {
            const userId = this.context.request.user.userId;
            try {
                const neb = yield NebStore_1.NebStore.createForAppservice(request.appserviceId, request.adminUrl);
                matrix_js_snippets_1.LogService.info("AdminNebService", userId + " created a new NEB instance from appservice " + request.appserviceId);
                MemoryCache_1.Cache.for(MemoryCache_1.CACHE_NEB).clear();
                MemoryCache_1.Cache.for(MemoryCache_1.CACHE_INTEGRATIONS).clear();
                return neb;
            }
            catch (err) {
                matrix_js_snippets_1.LogService.error("DimensionNebAdminService", err);
                throw new ApiError_1.ApiError(500, "Error creating go-neb instance");
            }
        });
    }
};
__decorate([
    typescript_rest_1.Context,
    __metadata("design:type", typescript_rest_1.ServiceContext)
], AdminNebService.prototype, "context", void 0);
__decorate([
    typescript_rest_1.GET,
    typescript_rest_1.Path("all"),
    typescript_rest_1.Security([MatrixSecurity_1.ROLE_USER, MatrixSecurity_1.ROLE_ADMIN]),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], AdminNebService.prototype, "getNebConfigs", null);
__decorate([
    typescript_rest_1.GET,
    typescript_rest_1.Path(":id/config"),
    typescript_rest_1.Security([MatrixSecurity_1.ROLE_USER, MatrixSecurity_1.ROLE_ADMIN]),
    __param(0, typescript_rest_1.PathParam("id")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Number]),
    __metadata("design:returntype", Promise)
], AdminNebService.prototype, "getNebConfig", null);
__decorate([
    typescript_rest_1.POST,
    typescript_rest_1.Path(":id/integration/:type/enabled"),
    typescript_rest_1.Security([MatrixSecurity_1.ROLE_USER, MatrixSecurity_1.ROLE_ADMIN]),
    __param(0, typescript_rest_1.PathParam("id")), __param(1, typescript_rest_1.PathParam("type")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Number, String, Object]),
    __metadata("design:returntype", Promise)
], AdminNebService.prototype, "setIntegrationEnabled", null);
__decorate([
    typescript_rest_1.POST,
    typescript_rest_1.Path(":id/integration/:type/config"),
    typescript_rest_1.Security([MatrixSecurity_1.ROLE_USER, MatrixSecurity_1.ROLE_ADMIN]),
    __param(0, typescript_rest_1.PathParam("id")), __param(1, typescript_rest_1.PathParam("type")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Number, String, Object]),
    __metadata("design:returntype", Promise)
], AdminNebService.prototype, "setIntegrationConfig", null);
__decorate([
    typescript_rest_1.GET,
    typescript_rest_1.Path(":id/integration/:type/config"),
    typescript_rest_1.Security([MatrixSecurity_1.ROLE_USER, MatrixSecurity_1.ROLE_ADMIN]),
    __param(0, typescript_rest_1.PathParam("id")), __param(1, typescript_rest_1.PathParam("type")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Number, String]),
    __metadata("design:returntype", Promise)
], AdminNebService.prototype, "getIntegrationConfig", null);
__decorate([
    typescript_rest_1.POST,
    typescript_rest_1.Path("new/upstream"),
    typescript_rest_1.Security([MatrixSecurity_1.ROLE_USER, MatrixSecurity_1.ROLE_ADMIN]),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], AdminNebService.prototype, "newConfigForUpstream", null);
__decorate([
    typescript_rest_1.POST,
    typescript_rest_1.Path("new/appservice"),
    typescript_rest_1.Security([MatrixSecurity_1.ROLE_USER, MatrixSecurity_1.ROLE_ADMIN]),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], AdminNebService.prototype, "newConfigForAppservice", null);
AdminNebService = __decorate([
    typescript_rest_1.Path("/api/v1/dimension/admin/neb")
], AdminNebService);
exports.AdminNebService = AdminNebService;
//# sourceMappingURL=AdminNebService.js.map