"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const typescript_rest_1 = require("typescript-rest");
const config_1 = require("../../config");
const MatrixLiteClient_1 = require("../../matrix/MatrixLiteClient");
const version_1 = require("../../version");
const helpers_1 = require("../../matrix/helpers");
const UserScalarToken_1 = require("../../db/models/UserScalarToken");
const MemoryCache_1 = require("../../MemoryCache");
const MatrixSecurity_1 = require("../security/MatrixSecurity");
/**
 * Administrative API for general information about Dimension
 */
let AdminService = class AdminService {
    checkIfAdmin() {
        return __awaiter(this, void 0, void 0, function* () {
            return {}; // A 200 OK essentially means "you're an admin".
        });
    }
    getVersion() {
        return __awaiter(this, void 0, void 0, function* () {
            return { version: version_1.CURRENT_VERSION };
        });
    }
    getConfig() {
        return __awaiter(this, void 0, void 0, function* () {
            const client = new MatrixLiteClient_1.MatrixLiteClient(config_1.default.homeserver.accessToken);
            const fedInfo = yield helpers_1.getFederationConnInfo(config_1.default.homeserver.name);
            return {
                admins: config_1.default.admins,
                widgetBlacklist: config_1.default.widgetBlacklist,
                homeserver: {
                    name: config_1.default.homeserver.name,
                    userId: yield client.whoAmI(),
                    federationUrl: fedInfo.url,
                    federationHostname: fedInfo.hostname,
                    clientServerUrl: config_1.default.homeserver.clientServerUrl,
                },
                sessionInfo: {
                    numTokens: yield UserScalarToken_1.default.count(),
                },
            };
        });
    }
    testFederationRouting(serverName) {
        return __awaiter(this, void 0, void 0, function* () {
            return {
                inputServerName: serverName,
                resolvedServer: yield helpers_1.getFederationConnInfo(serverName),
            };
        });
    }
    logoutAll() {
        return __awaiter(this, void 0, void 0, function* () {
            // Clear the cache first to hopefully invalidate a bunch of them
            MemoryCache_1.Cache.for(MemoryCache_1.CACHE_SCALAR_ACCOUNTS).clear();
            const tokens = yield UserScalarToken_1.default.findAll();
            for (const token of tokens) {
                yield token.destroy();
            }
            // Clear it again because the delete loop can be slow
            MemoryCache_1.Cache.for(MemoryCache_1.CACHE_SCALAR_ACCOUNTS).clear();
            return {};
        });
    }
};
__decorate([
    typescript_rest_1.GET,
    typescript_rest_1.Path("check"),
    typescript_rest_1.Security([MatrixSecurity_1.ROLE_USER, MatrixSecurity_1.ROLE_ADMIN]),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], AdminService.prototype, "checkIfAdmin", null);
__decorate([
    typescript_rest_1.GET,
    typescript_rest_1.Path("version"),
    typescript_rest_1.Security([MatrixSecurity_1.ROLE_USER, MatrixSecurity_1.ROLE_ADMIN]),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], AdminService.prototype, "getVersion", null);
__decorate([
    typescript_rest_1.GET,
    typescript_rest_1.Path("config"),
    typescript_rest_1.Security([MatrixSecurity_1.ROLE_USER, MatrixSecurity_1.ROLE_ADMIN]),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], AdminService.prototype, "getConfig", null);
__decorate([
    typescript_rest_1.GET,
    typescript_rest_1.Path("test/federation"),
    typescript_rest_1.Security([MatrixSecurity_1.ROLE_USER, MatrixSecurity_1.ROLE_ADMIN]),
    __param(0, typescript_rest_1.QueryParam("server_name")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", Promise)
], AdminService.prototype, "testFederationRouting", null);
__decorate([
    typescript_rest_1.POST,
    typescript_rest_1.Path("sessions/logout/all"),
    typescript_rest_1.Security([MatrixSecurity_1.ROLE_USER, MatrixSecurity_1.ROLE_ADMIN]),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], AdminService.prototype, "logoutAll", null);
AdminService = __decorate([
    typescript_rest_1.Path("/api/v1/dimension/admin")
], AdminService);
exports.AdminService = AdminService;
//# sourceMappingURL=AdminService.js.map