"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const typescript_rest_1 = require("typescript-rest");
const StickerPack_1 = require("../../db/models/StickerPack");
const ApiError_1 = require("../ApiError");
const DimensionStickerService_1 = require("../dimension/DimensionStickerService");
const MemoryCache_1 = require("../../MemoryCache");
const TelegramBot_1 = require("../../utils/TelegramBot");
const MatrixLiteClient_1 = require("../../matrix/MatrixLiteClient");
const config_1 = require("../../config");
const Sticker_1 = require("../../db/models/Sticker");
const matrix_js_snippets_1 = require("matrix-js-snippets");
const sharp = require("sharp");
const MatrixSecurity_1 = require("../security/MatrixSecurity");
/**
 * Administrative API for configuring stickers
 */
let AdminStickerService = class AdminStickerService {
    getStickerPacks() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield DimensionStickerService_1.DimensionStickerService.getStickerPacks(false);
        });
    }
    setPackEnabled(packId, request) {
        return __awaiter(this, void 0, void 0, function* () {
            const pack = yield StickerPack_1.default.findByPk(packId);
            if (!pack)
                throw new ApiError_1.ApiError(404, "Sticker pack not found");
            pack.isEnabled = request.isEnabled;
            yield pack.save();
            MemoryCache_1.Cache.for(MemoryCache_1.CACHE_STICKERS).clear();
            return {}; // 200 OK
        });
    }
    importFromTelegram(request) {
        return __awaiter(this, void 0, void 0, function* () {
            const userId = this.context.request.user.userId;
            if (!request.packUrl || (!request.packUrl.startsWith("https://t.me/addstickers/") && !request.packUrl.startsWith("https://telegram.me/addstickers/"))) {
                throw new ApiError_1.ApiError(400, "Invalid pack URL");
            }
            const tg = new TelegramBot_1.TelegramBot();
            const mx = new MatrixLiteClient_1.MatrixLiteClient(config_1.default.homeserver.accessToken);
            matrix_js_snippets_1.LogService.info("AdminStickerService", "Importing " + request.packUrl + " from Telegram for " + userId);
            const tgPack = yield tg.getStickerPack(request.packUrl);
            const pack = yield StickerPack_1.default.create({
                type: "stickerpack",
                name: tgPack.name,
                description: tgPack.description,
                avatarUrl: "mxc://localhost/NotYetLoaded",
                isEnabled: false,
                isPublic: true,
                authorType: "telegram",
                authorName: "Telegram",
                authorReference: request.packUrl,
                license: "Telegram",
                licensePath: "/licenses/telegram-imported.txt",
            });
            const stickers = [];
            try {
                let avatarUrl = null;
                for (const tgSticker of tgPack.stickers) {
                    matrix_js_snippets_1.LogService.info("AdminStickerService", "Importing sticker from " + tgSticker.url);
                    const buffer = yield mx.downloadFromUrl(tgSticker.url);
                    const png = yield sharp(buffer).resize({
                        width: 512,
                        height: 512,
                        fit: 'contain',
                        background: 'rgba(0,0,0,0)',
                    }).png().toBuffer();
                    const mxc = yield mx.upload(png, "image/png");
                    const serverName = mxc.substring("mxc://".length).split("/")[0];
                    const contentId = mxc.substring("mxc://".length).split("/")[1];
                    const thumbMxc = yield mx.uploadFromUrl(yield mx.getThumbnailUrl(serverName, contentId, 512, 512, "scale", false), "image/png");
                    stickers.push(yield Sticker_1.default.create({
                        packId: pack.id,
                        name: tgSticker.emoji,
                        description: tgSticker.emoji,
                        imageMxc: mxc,
                        thumbnailMxc: thumbMxc,
                        thumbnailWidth: 512,
                        thumbnailHeight: 512,
                        mimetype: "image/png",
                    }));
                    if (!avatarUrl)
                        avatarUrl = mxc;
                }
                pack.avatarUrl = avatarUrl;
                yield pack.save();
            }
            catch (e) {
                matrix_js_snippets_1.LogService.error("AdminStickerService", e);
                const thingsToDelete = [pack, ...stickers];
                for (const item of thingsToDelete) {
                    try {
                        yield item.destroy();
                    }
                    catch (e2) {
                        matrix_js_snippets_1.LogService.error("AdminStickerService", e2);
                    }
                }
                throw new ApiError_1.ApiError(500, "Error importing stickers");
            }
            MemoryCache_1.Cache.for(MemoryCache_1.CACHE_STICKERS).clear();
            return DimensionStickerService_1.DimensionStickerService.packToMemory(pack);
        });
    }
};
__decorate([
    typescript_rest_1.Context,
    __metadata("design:type", typescript_rest_1.ServiceContext)
], AdminStickerService.prototype, "context", void 0);
__decorate([
    typescript_rest_1.GET,
    typescript_rest_1.Path("packs"),
    typescript_rest_1.Security([MatrixSecurity_1.ROLE_USER, MatrixSecurity_1.ROLE_ADMIN]),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], AdminStickerService.prototype, "getStickerPacks", null);
__decorate([
    typescript_rest_1.POST,
    typescript_rest_1.Path("packs/:id/enabled"),
    typescript_rest_1.Security([MatrixSecurity_1.ROLE_USER, MatrixSecurity_1.ROLE_ADMIN]),
    __param(0, typescript_rest_1.PathParam("id")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Number, Object]),
    __metadata("design:returntype", Promise)
], AdminStickerService.prototype, "setPackEnabled", null);
__decorate([
    typescript_rest_1.POST,
    typescript_rest_1.Path("packs/import/telegram"),
    typescript_rest_1.Security([MatrixSecurity_1.ROLE_USER, MatrixSecurity_1.ROLE_ADMIN]),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], AdminStickerService.prototype, "importFromTelegram", null);
AdminStickerService = __decorate([
    typescript_rest_1.Path("/api/v1/dimension/admin/stickers")
], AdminStickerService);
exports.AdminStickerService = AdminStickerService;
//# sourceMappingURL=AdminStickerService.js.map