"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const typescript_rest_1 = require("typescript-rest");
const MemoryCache_1 = require("../../MemoryCache");
const Upstream_1 = require("../../db/models/Upstream");
const matrix_js_snippets_1 = require("matrix-js-snippets");
const MatrixSecurity_1 = require("../security/MatrixSecurity");
/**
 * Administrative API for managing the instances upstream of this instance. Particularly
 * useful for configuring the Modular upstream.
 */
let AdminUpstreamService = class AdminUpstreamService {
    getUpstreams() {
        return __awaiter(this, void 0, void 0, function* () {
            const cachedUpstreams = MemoryCache_1.Cache.for(MemoryCache_1.CACHE_UPSTREAM).get("upstreams");
            if (cachedUpstreams)
                return cachedUpstreams;
            const upstreams = yield Upstream_1.default.findAll();
            const mapped = upstreams.map(u => this.mapUpstream(u));
            MemoryCache_1.Cache.for(MemoryCache_1.CACHE_UPSTREAM).put("upstreams", mapped);
            return mapped;
        });
    }
    createUpstream(request) {
        return __awaiter(this, void 0, void 0, function* () {
            const userId = this.context.request.user.userId;
            const upstream = yield Upstream_1.default.create({
                name: request.name,
                type: request.type,
                scalarUrl: request.scalarUrl,
                apiUrl: request.apiUrl,
            });
            matrix_js_snippets_1.LogService.info("AdminUpstreamService", userId + " created a new upstream");
            MemoryCache_1.Cache.for(MemoryCache_1.CACHE_UPSTREAM).clear();
            MemoryCache_1.Cache.for(MemoryCache_1.CACHE_SCALAR_ACCOUNTS).clear();
            return this.mapUpstream(upstream);
        });
    }
    mapUpstream(upstream) {
        return {
            id: upstream.id,
            name: upstream.name,
            type: upstream.type,
            scalarUrl: upstream.scalarUrl,
            apiUrl: upstream.apiUrl
        };
    }
};
__decorate([
    typescript_rest_1.Context,
    __metadata("design:type", typescript_rest_1.ServiceContext)
], AdminUpstreamService.prototype, "context", void 0);
__decorate([
    typescript_rest_1.GET,
    typescript_rest_1.Path("all"),
    typescript_rest_1.Security([MatrixSecurity_1.ROLE_USER, MatrixSecurity_1.ROLE_ADMIN]),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], AdminUpstreamService.prototype, "getUpstreams", null);
__decorate([
    typescript_rest_1.POST,
    typescript_rest_1.Path("new"),
    typescript_rest_1.Security([MatrixSecurity_1.ROLE_USER, MatrixSecurity_1.ROLE_ADMIN]),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], AdminUpstreamService.prototype, "createUpstream", null);
AdminUpstreamService = __decorate([
    typescript_rest_1.Path("/api/v1/dimension/admin/upstreams")
], AdminUpstreamService);
exports.AdminUpstreamService = AdminUpstreamService;
//# sourceMappingURL=AdminUpstreamService.js.map