"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const typescript_rest_1 = require("typescript-rest");
const MemoryCache_1 = require("../../MemoryCache");
const matrix_js_snippets_1 = require("matrix-js-snippets");
const ApiError_1 = require("../ApiError");
const WebhookBridgeRecord_1 = require("../../db/models/WebhookBridgeRecord");
const MatrixSecurity_1 = require("../security/MatrixSecurity");
/**
 * Administrative API for configuring Webhook bridge instances.
 */
let AdminWebhooksService = class AdminWebhooksService {
    getBridges() {
        return __awaiter(this, void 0, void 0, function* () {
            const bridges = yield WebhookBridgeRecord_1.default.findAll();
            return Promise.all(bridges.map((b) => __awaiter(this, void 0, void 0, function* () {
                return {
                    id: b.id,
                    upstreamId: b.upstreamId,
                    provisionUrl: b.provisionUrl,
                    sharedSecret: b.sharedSecret,
                    isEnabled: b.isEnabled,
                };
            })));
        });
    }
    getBridge(bridgeId) {
        return __awaiter(this, void 0, void 0, function* () {
            const webhookBridge = yield WebhookBridgeRecord_1.default.findByPk(bridgeId);
            if (!webhookBridge)
                throw new ApiError_1.ApiError(404, "Webhook Bridge not found");
            return {
                id: webhookBridge.id,
                upstreamId: webhookBridge.upstreamId,
                provisionUrl: webhookBridge.provisionUrl,
                sharedSecret: webhookBridge.sharedSecret,
                isEnabled: webhookBridge.isEnabled,
            };
        });
    }
    updateBridge(bridgeId, request) {
        return __awaiter(this, void 0, void 0, function* () {
            const userId = this.context.request.user.userId;
            const bridge = yield WebhookBridgeRecord_1.default.findByPk(bridgeId);
            if (!bridge)
                throw new ApiError_1.ApiError(404, "Bridge not found");
            bridge.provisionUrl = request.provisionUrl;
            bridge.sharedSecret = request.sharedSecret;
            yield bridge.save();
            matrix_js_snippets_1.LogService.info("AdminWebhooksService", userId + " updated Webhook Bridge " + bridge.id);
            MemoryCache_1.Cache.for(MemoryCache_1.CACHE_WEBHOOKS_BRIDGE).clear();
            MemoryCache_1.Cache.for(MemoryCache_1.CACHE_INTEGRATIONS).clear();
            return this.getBridge(bridge.id);
        });
    }
    newConfigForUpstream(_scalarToken, _request) {
        return __awaiter(this, void 0, void 0, function* () {
            throw new ApiError_1.ApiError(400, "Cannot create a webhook bridge from an upstream");
        });
    }
    newSelfhosted(request) {
        return __awaiter(this, void 0, void 0, function* () {
            const userId = this.context.request.user.userId;
            const bridge = yield WebhookBridgeRecord_1.default.create({
                provisionUrl: request.provisionUrl,
                sharedSecret: request.sharedSecret,
                isEnabled: true,
            });
            matrix_js_snippets_1.LogService.info("AdminWebhooksService", userId + " created a new Webhook Bridge with provisioning URL " + request.provisionUrl);
            MemoryCache_1.Cache.for(MemoryCache_1.CACHE_WEBHOOKS_BRIDGE).clear();
            MemoryCache_1.Cache.for(MemoryCache_1.CACHE_INTEGRATIONS).clear();
            return this.getBridge(bridge.id);
        });
    }
};
__decorate([
    typescript_rest_1.Context,
    __metadata("design:type", typescript_rest_1.ServiceContext)
], AdminWebhooksService.prototype, "context", void 0);
__decorate([
    typescript_rest_1.GET,
    typescript_rest_1.Path("all"),
    typescript_rest_1.Security([MatrixSecurity_1.ROLE_USER, MatrixSecurity_1.ROLE_ADMIN]),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], AdminWebhooksService.prototype, "getBridges", null);
__decorate([
    typescript_rest_1.GET,
    typescript_rest_1.Path(":bridgeId"),
    typescript_rest_1.Security([MatrixSecurity_1.ROLE_USER, MatrixSecurity_1.ROLE_ADMIN]),
    __param(0, typescript_rest_1.PathParam("bridgeId")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Number]),
    __metadata("design:returntype", Promise)
], AdminWebhooksService.prototype, "getBridge", null);
__decorate([
    typescript_rest_1.POST,
    typescript_rest_1.Path(":bridgeId"),
    typescript_rest_1.Security([MatrixSecurity_1.ROLE_USER, MatrixSecurity_1.ROLE_ADMIN]),
    __param(0, typescript_rest_1.PathParam("bridgeId")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Number, Object]),
    __metadata("design:returntype", Promise)
], AdminWebhooksService.prototype, "updateBridge", null);
__decorate([
    typescript_rest_1.POST,
    typescript_rest_1.Path("new/upstream"),
    typescript_rest_1.Security([MatrixSecurity_1.ROLE_USER, MatrixSecurity_1.ROLE_ADMIN]),
    __param(0, typescript_rest_1.QueryParam("scalar_token")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, Object]),
    __metadata("design:returntype", Promise)
], AdminWebhooksService.prototype, "newConfigForUpstream", null);
__decorate([
    typescript_rest_1.POST,
    typescript_rest_1.Path("new/selfhosted"),
    typescript_rest_1.Security([MatrixSecurity_1.ROLE_USER, MatrixSecurity_1.ROLE_ADMIN]),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], AdminWebhooksService.prototype, "newSelfhosted", null);
AdminWebhooksService = __decorate([
    typescript_rest_1.Path("/api/v1/dimension/admin/webhooks")
], AdminWebhooksService);
exports.AdminWebhooksService = AdminWebhooksService;
//# sourceMappingURL=AdminWebhooksService.js.map