"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const es6_1 = require("typescript-ioc/es6");
const TermsRecord_1 = require("../../db/models/TermsRecord");
const TermsTextRecord_1 = require("../../db/models/TermsTextRecord");
const TermsSignedRecord_1 = require("../../db/models/TermsSignedRecord");
const sequelize_1 = require("sequelize");
const MemoryCache_1 = require("../../MemoryCache");
const UserScalarToken_1 = require("../../db/models/UserScalarToken");
const Upstream_1 = require("../../db/models/Upstream");
const matrix_js_snippets_1 = require("matrix-js-snippets");
const ScalarClient_1 = require("../../scalar/ScalarClient");
const hashing_1 = require("../../utils/hashing");
const TermsUpstreamRecord_1 = require("../../db/models/TermsUpstreamRecord");
const ScalarStore_1 = require("../../db/ScalarStore");
exports.VERSION_DRAFT = "draft";
/**
 * API controller for terms of service management
 */
let TermsController = class TermsController {
    constructor() {
    }
    getPublishedTerms() {
        return __awaiter(this, void 0, void 0, function* () {
            const cache = MemoryCache_1.Cache.for(MemoryCache_1.CACHE_TERMS);
            let terms = cache.get("published");
            if (terms)
                return terms;
            terms = (yield TermsRecord_1.default.findAll({
                where: { version: { [sequelize_1.Op.ne]: exports.VERSION_DRAFT } },
                include: [TermsTextRecord_1.default],
            }));
            const latest = {};
            for (const record of terms) {
                if (!latest[record.shortcode]) {
                    latest[record.shortcode] = record;
                }
                if (latest[record.shortcode].id < record.id) {
                    latest[record.shortcode] = record;
                }
            }
            terms = Object.values(latest).map(p => {
                const mapped = this.mapPolicy(false, p);
                return Object.assign({}, mapped, {
                    id: p.id,
                });
            });
            cache.put("published", terms);
            return terms;
        });
    }
    doesUserNeedToSignTerms(user) {
        return __awaiter(this, void 0, void 0, function* () {
            const latest = yield this.getPublishedTerms();
            const signed = yield TermsSignedRecord_1.default.findAll({ where: { userId: user.userId } });
            const missing = Object.values(latest).filter(d => !signed.find(s => s.termsId === d.id));
            if (missing.length > 0)
                return true;
            // Test upstream terms for the user
            const tokensForUser = yield UserScalarToken_1.default.findAll({ where: { userId: user.userId }, include: [Upstream_1.default] });
            const upstreamTokens = tokensForUser.filter(t => t.upstream);
            for (const upstreamToken of upstreamTokens) {
                try {
                    const online = yield ScalarStore_1.ScalarStore.isUpstreamOnline(upstreamToken.upstream, ScalarClient_1.ScalarClient.KIND_MATRIX_V1);
                    if (!online) {
                        matrix_js_snippets_1.LogService.warn("TermsController", `Upstream ${upstreamToken.upstream.id} is offline - skipping terms check`);
                        continue;
                    }
                    const scalarClient = new ScalarClient_1.ScalarClient(upstreamToken.upstream, ScalarClient_1.ScalarClient.KIND_MATRIX_V1);
                    yield scalarClient.getAccount(upstreamToken.scalarToken);
                    // 200 OK means we're fine
                }
                catch (e) {
                    if (e.statusCode === 403 && e.body && e.body.errcode === 'M_TERMS_NOT_SIGNED') {
                        return true;
                    }
                }
            }
            return false;
        });
    }
    getAvailableTerms() {
        return __awaiter(this, void 0, void 0, function* () {
            const latest = yield this.getPublishedTerms();
            const policies = { policies: {} };
            for (const termsPolicy of Object.values(latest)) {
                policies.policies[termsPolicy.shortcode] = {
                    version: termsPolicy.version,
                };
                for (const language in termsPolicy.languages) {
                    policies.policies[termsPolicy.shortcode][language] = {
                        name: termsPolicy.languages[language].name,
                        url: termsPolicy.languages[language].url,
                    };
                }
            }
            // Get upstream terms
            const usptreams = yield Upstream_1.default.findAll();
            const urlsToUpstream = {}; // {url: [upstreamId]}
            for (const upstream of usptreams) {
                try {
                    const scalarClient = new ScalarClient_1.ScalarClient(upstream, ScalarClient_1.ScalarClient.KIND_MATRIX_V1);
                    const upstreamTerms = yield scalarClient.getAvailableTerms();
                    // rewrite the shortcodes to avoid conflicts
                    const shortcodePrefix = `upstream_${hashing_1.md5(`${upstream.id}:${upstream.apiUrl}`)}`;
                    for (const shortcode of Object.keys(upstreamTerms.policies)) {
                        policies.policies[`${shortcodePrefix}_${shortcode}`] = upstreamTerms.policies[shortcode];
                        // copy all urls for later adding to the database
                        for (const language of Object.keys(upstreamTerms.policies[shortcode])) {
                            const upstreamUrl = upstreamTerms.policies[shortcode][language]['url'];
                            if (!urlsToUpstream[upstreamUrl])
                                urlsToUpstream[upstreamUrl] = [];
                            const upstreamsArr = urlsToUpstream[upstreamUrl];
                            if (!upstreamsArr.includes(upstream.id)) {
                                upstreamsArr.push(upstream.id);
                            }
                        }
                    }
                }
                catch (e) {
                    matrix_js_snippets_1.LogService.error("TermsController", e);
                }
            }
            // actually cache the urls in the database
            const existingCache = yield TermsUpstreamRecord_1.default.findAll({ where: { url: { [sequelize_1.Op.in]: Object.keys(urlsToUpstream) } } });
            for (const upstreamUrl of Object.keys(urlsToUpstream)) {
                const upstreamIds = urlsToUpstream[upstreamUrl];
                const existingIds = existingCache.filter(c => c.url === upstreamUrl).map(c => c.upstreamId);
                const missingIds = upstreamIds.filter(i => !existingIds.includes(i));
                for (const targetUpstreamId of missingIds) {
                    const item = yield TermsUpstreamRecord_1.default.create({ url: upstreamUrl, upstreamId: targetUpstreamId });
                    existingCache.push(item);
                }
            }
            return policies;
        });
    }
    signTermsMatching(user, urls) {
        return __awaiter(this, void 0, void 0, function* () {
            const terms = yield TermsTextRecord_1.default.findAll({ where: { url: { [sequelize_1.Op.in]: urls } } });
            const signed = yield TermsSignedRecord_1.default.findAll({ where: { userId: user.userId } });
            const toAdd = terms.filter(t => !signed.find(s => s.termsId === t.termsId));
            for (const termsToSign of toAdd) {
                yield TermsSignedRecord_1.default.create({ termsId: termsToSign.id, userId: user.userId });
            }
            // Check upstreams too, if there are any
            const upstreamPolicies = yield TermsUpstreamRecord_1.default.findAll({
                where: { url: { [sequelize_1.Op.in]: urls } },
                include: [Upstream_1.default]
            });
            const upstreamsToSignatures = {};
            for (const upstreamPolicy of upstreamPolicies) {
                const userToken = yield UserScalarToken_1.default.findOne({
                    where: {
                        upstreamId: upstreamPolicy.upstreamId,
                        userId: user.userId,
                    },
                });
                if (!userToken) {
                    matrix_js_snippets_1.LogService.warn("TermsController", `User ${user.userId} is missing an upstream token for ${upstreamPolicy.upstream.scalarUrl}`);
                    continue;
                }
                if (!upstreamsToSignatures[upstreamPolicy.upstreamId])
                    upstreamsToSignatures[upstreamPolicy.upstreamId] = {
                        upstream: upstreamPolicy.upstream,
                        token: userToken.scalarToken,
                        urls: [],
                    };
                upstreamsToSignatures[upstreamPolicy.upstreamId].urls.push(upstreamPolicy.url);
            }
            for (const upstreamSignature of Object.values(upstreamsToSignatures)) {
                const client = new ScalarClient_1.ScalarClient(upstreamSignature.upstream, ScalarClient_1.ScalarClient.KIND_MATRIX_V1);
                yield client.signTermsUrls(upstreamSignature.token, upstreamSignature.urls);
            }
        });
    }
    getPoliciesForAdmin() {
        return __awaiter(this, void 0, void 0, function* () {
            const terms = yield TermsRecord_1.default.findAll({ include: [TermsTextRecord_1.default] });
            return terms.map(this.mapPolicy.bind(this, false));
        });
    }
    getPolicyForAdmin(shortcode, version) {
        return __awaiter(this, void 0, void 0, function* () {
            const terms = yield TermsRecord_1.default.findOne({ where: { shortcode, version }, include: [TermsTextRecord_1.default] });
            return this.mapPolicy(true, terms);
        });
    }
    createDraftPolicy(name, shortcode, text, url) {
        return __awaiter(this, void 0, void 0, function* () {
            const terms = yield TermsRecord_1.default.create({ shortcode, version: exports.VERSION_DRAFT });
            const termsText = yield TermsTextRecord_1.default.create({ termsId: terms.id, language: "en", name, text, url });
            terms.texts = [termsText];
            return this.mapPolicy(true, terms);
        });
    }
    updatePolicy(name, shortcode, version, text, url) {
        return __awaiter(this, void 0, void 0, function* () {
            const terms = yield TermsRecord_1.default.findOne({ where: { shortcode, version }, include: [TermsTextRecord_1.default] });
            const termsText = terms.texts.find(e => e.language === "en");
            termsText.url = url;
            termsText.text = text;
            termsText.name = name;
            yield termsText.save();
            MemoryCache_1.Cache.for(MemoryCache_1.CACHE_TERMS).clear();
            return this.mapPolicy(true, terms);
        });
    }
    publishPolicy(shortcode, targetVersion) {
        return __awaiter(this, void 0, void 0, function* () {
            const terms = yield TermsRecord_1.default.findOne({
                where: { shortcode, version: exports.VERSION_DRAFT },
                include: [TermsTextRecord_1.default],
            });
            if (!terms)
                throw new Error("Missing terms");
            terms.version = targetVersion;
            yield terms.save();
            MemoryCache_1.Cache.for(MemoryCache_1.CACHE_TERMS).clear();
            return this.mapPolicy(true, terms);
        });
    }
    mapPolicy(withText, policy) {
        const languages = {};
        policy.texts.forEach(pt => languages[pt.language] = {
            name: pt.name,
            url: pt.url,
            text: withText ? pt.text : null,
        });
        return {
            shortcode: policy.shortcode,
            version: policy.version,
            languages: languages,
        };
    }
};
TermsController = __decorate([
    es6_1.AutoWired,
    __metadata("design:paramtypes", [])
], TermsController);
exports.default = TermsController;
//# sourceMappingURL=TermsController.js.map