"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const typescript_rest_1 = require("typescript-rest");
const ApiError_1 = require("../ApiError");
const GitterBridge_1 = require("../../bridges/GitterBridge");
const matrix_js_snippets_1 = require("matrix-js-snippets");
const MatrixSecurity_1 = require("../security/MatrixSecurity");
/**
 * API for interacting with the Gitter bridge
 */
let DimensionGitterService = class DimensionGitterService {
    getLink(roomId) {
        return __awaiter(this, void 0, void 0, function* () {
            const userId = this.context.request.user.userId;
            try {
                const gitter = new GitterBridge_1.GitterBridge(userId);
                return gitter.getLink(roomId);
            }
            catch (e) {
                matrix_js_snippets_1.LogService.error("DimensionGitterService", e);
                throw new ApiError_1.ApiError(400, "Error getting bridge info");
            }
        });
    }
    bridgeRoom(roomId, request) {
        return __awaiter(this, void 0, void 0, function* () {
            const userId = this.context.request.user.userId;
            try {
                const gitter = new GitterBridge_1.GitterBridge(userId);
                yield gitter.requestLink(roomId, request.gitterRoomName);
                return gitter.getLink(roomId);
            }
            catch (e) {
                matrix_js_snippets_1.LogService.error("DimensionGitterService", e);
                throw new ApiError_1.ApiError(400, "Error bridging room");
            }
        });
    }
    unbridgeRoom(roomId) {
        return __awaiter(this, void 0, void 0, function* () {
            const userId = this.context.request.user.userId;
            try {
                const gitter = new GitterBridge_1.GitterBridge(userId);
                const link = yield gitter.getLink(roomId);
                yield gitter.removeLink(roomId, link.gitterRoomName);
                return {}; // 200 OK
            }
            catch (e) {
                matrix_js_snippets_1.LogService.error("DimensionGitterService", e);
                throw new ApiError_1.ApiError(400, "Error unbridging room");
            }
        });
    }
};
__decorate([
    typescript_rest_1.Context,
    __metadata("design:type", typescript_rest_1.ServiceContext)
], DimensionGitterService.prototype, "context", void 0);
__decorate([
    typescript_rest_1.GET,
    typescript_rest_1.Path("room/:roomId/link"),
    typescript_rest_1.Security(MatrixSecurity_1.ROLE_USER),
    __param(0, typescript_rest_1.PathParam("roomId")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", Promise)
], DimensionGitterService.prototype, "getLink", null);
__decorate([
    typescript_rest_1.POST,
    typescript_rest_1.Path("room/:roomId/link"),
    typescript_rest_1.Security(MatrixSecurity_1.ROLE_USER),
    __param(0, typescript_rest_1.PathParam("roomId")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, Object]),
    __metadata("design:returntype", Promise)
], DimensionGitterService.prototype, "bridgeRoom", null);
__decorate([
    typescript_rest_1.DELETE,
    typescript_rest_1.Path("room/:roomId/link"),
    typescript_rest_1.Security(MatrixSecurity_1.ROLE_USER),
    __param(0, typescript_rest_1.PathParam("roomId")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", Promise)
], DimensionGitterService.prototype, "unbridgeRoom", null);
DimensionGitterService = __decorate([
    typescript_rest_1.Path("/api/v1/dimension/gitter")
], DimensionGitterService);
exports.DimensionGitterService = DimensionGitterService;
//# sourceMappingURL=DimensionGitterService.js.map