"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var DimensionIntegrationsService_1;
Object.defineProperty(exports, "__esModule", { value: true });
const typescript_rest_1 = require("typescript-rest");
const MemoryCache_1 = require("../../MemoryCache");
const ApiError_1 = require("../ApiError");
const WidgetStore_1 = require("../../db/WidgetStore");
const SimpleBot_1 = require("../../integrations/SimpleBot");
const NebStore_1 = require("../../db/NebStore");
const BridgeStore_1 = require("../../db/BridgeStore");
const BotStore_1 = require("../../db/BotStore");
const MatrixSecurity_1 = require("../security/MatrixSecurity");
/**
 * API for managing integrations, primarily for a given room
 */
let DimensionIntegrationsService = DimensionIntegrationsService_1 = class DimensionIntegrationsService {
    /**
     * Gets a list of widgets
     * @param {boolean} enabledOnly True to only return the enabled widgets
     * @returns {Promise<Widget[]>} Resolves to the widget list
     */
    static getWidgets(enabledOnly) {
        return __awaiter(this, void 0, void 0, function* () {
            const cached = MemoryCache_1.Cache.for(MemoryCache_1.CACHE_INTEGRATIONS).get("widgets");
            if (cached)
                return cached;
            const widgets = yield WidgetStore_1.WidgetStore.listAll(enabledOnly ? true : null);
            MemoryCache_1.Cache.for(MemoryCache_1.CACHE_INTEGRATIONS).put("widgets", widgets);
            return widgets;
        });
    }
    /**
     * Gets a list of bridges
     * @param {boolean} enabledOnly True to only return the enabled bridges
     * @param {string} forUserId The requesting user ID
     * @param {string} inRoomId If specified, the room ID to list the bridges in
     * @returns {Promise<Bridge[]>} Resolves to the bridge list
     */
    static getBridges(enabledOnly, forUserId, inRoomId) {
        return __awaiter(this, void 0, void 0, function* () {
            return BridgeStore_1.BridgeStore.listAll(forUserId, enabledOnly ? true : null, inRoomId);
        });
    }
    /**
     * Gets a list of simple bots
     * @param {string} userId The requesting user ID
     * @returns {Promise<SimpleBot[]>} Resolves to the simple bot list
     */
    static getSimpleBots(userId) {
        return __awaiter(this, void 0, void 0, function* () {
            const cached = MemoryCache_1.Cache.for(MemoryCache_1.CACHE_INTEGRATIONS).get("simple_bots");
            if (cached)
                return cached;
            const nebs = yield NebStore_1.NebStore.listSimpleBots(userId);
            const custom = (yield BotStore_1.BotStore.getCustomBots())
                .filter(b => b.isEnabled)
                .map(b => SimpleBot_1.SimpleBot.fromCached(b));
            const bots = [...nebs, ...custom];
            MemoryCache_1.Cache.for(MemoryCache_1.CACHE_INTEGRATIONS).put("simple_bots", bots);
            return bots;
        });
    }
    /**
     * Gets a list of complex bots
     * @param {string} userId The requesting user ID
     * @param {string} roomId The room ID to get the complex bots for
     * @returns {Promise<ComplexBot[]>} Resolves to the complex bot list
     */
    static getComplexBots(userId, roomId) {
        return __awaiter(this, void 0, void 0, function* () {
            const cached = MemoryCache_1.Cache.for(MemoryCache_1.CACHE_INTEGRATIONS).get("complex_bots_" + roomId);
            if (cached)
                return cached;
            const bots = yield NebStore_1.NebStore.listComplexBots(userId, roomId);
            MemoryCache_1.Cache.for(MemoryCache_1.CACHE_INTEGRATIONS).put("complex_bots_" + roomId, bots);
            return bots;
        });
    }
    getIntegrationsInRoom(roomId) {
        return __awaiter(this, void 0, void 0, function* () {
            const userId = this.context.request.user.userId;
            return {
                widgets: yield DimensionIntegrationsService_1.getWidgets(true),
                bots: yield DimensionIntegrationsService_1.getSimpleBots(userId),
                complexBots: yield DimensionIntegrationsService_1.getComplexBots(userId, roomId),
                bridges: yield DimensionIntegrationsService_1.getBridges(true, userId, roomId),
            };
        });
    }
    getIntegrationInRoom(roomId, category, integrationType) {
        return __awaiter(this, void 0, void 0, function* () {
            const roomConfig = yield this.getIntegrationsInRoom(roomId); // does auth for us
            if (category === "widget")
                return roomConfig.widgets.find(i => i.type === integrationType);
            else if (category === "bot")
                return roomConfig.bots.find(i => i.type === integrationType);
            else if (category === "complex-bot")
                return roomConfig.complexBots.find(i => i.type === integrationType);
            else if (category === "bridge")
                return roomConfig.bridges.find(i => i.type === integrationType);
            else
                throw new ApiError_1.ApiError(400, "Unrecognized category");
        });
    }
    setIntegrationConfigurationInRoom(roomId, category, integrationType, newConfig) {
        return __awaiter(this, void 0, void 0, function* () {
            const userId = this.context.request.user.userId;
            if (category === "complex-bot")
                yield NebStore_1.NebStore.setComplexBotConfig(userId, integrationType, roomId, newConfig);
            else if (category === "bridge")
                yield BridgeStore_1.BridgeStore.setBridgeRoomConfig(userId, integrationType, roomId, newConfig);
            else
                throw new ApiError_1.ApiError(400, "Unrecognized category");
            MemoryCache_1.Cache.for(MemoryCache_1.CACHE_INTEGRATIONS).clear(); // TODO: Improve which cache we invalidate
            return {}; // 200 OK
        });
    }
    removeIntegrationInRoom(roomId, category, integrationType) {
        return __awaiter(this, void 0, void 0, function* () {
            const userId = this.context.request.user.userId;
            if (category === "widget")
                throw new ApiError_1.ApiError(400, "Widgets should be removed client-side");
            else if (category === "bot") {
                if (integrationType.startsWith(BotStore_1.BotStore.TYPE_PREFIX)) {
                    yield BotStore_1.BotStore.removeCustomByTypeFromRoom(integrationType, roomId);
                }
                else
                    yield NebStore_1.NebStore.removeSimpleBot(integrationType, roomId, userId);
            }
            else if (category === "complex-bot")
                throw new ApiError_1.ApiError(400, "Complex bots should be removed automatically");
            else if (category === "bridge")
                throw new ApiError_1.ApiError(400, "Bridges should be removed automatically");
            else
                throw new ApiError_1.ApiError(400, "Unrecognized category");
            MemoryCache_1.Cache.for(MemoryCache_1.CACHE_INTEGRATIONS).clear(); // TODO: Improve which cache we invalidate
            return {}; // 200 OK
        });
    }
    getIntegration(category, type) {
        return __awaiter(this, void 0, void 0, function* () {
            // This is intentionally an unauthed endpoint to ensure we can use it in widgets
            let integrations = [];
            if (category === "widget")
                integrations = yield DimensionIntegrationsService_1.getWidgets(true);
            else
                throw new ApiError_1.ApiError(400, "Unsupported category");
            for (const integration of integrations) {
                if (integration.category === category && integration.type === type) {
                    return integration;
                }
            }
            throw new ApiError_1.ApiError(404, "Integration not found");
        });
    }
};
__decorate([
    typescript_rest_1.Context,
    __metadata("design:type", typescript_rest_1.ServiceContext)
], DimensionIntegrationsService.prototype, "context", void 0);
__decorate([
    typescript_rest_1.GET,
    typescript_rest_1.Path("room/:roomId"),
    typescript_rest_1.Security(MatrixSecurity_1.ROLE_USER),
    __param(0, typescript_rest_1.PathParam("roomId")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", Promise)
], DimensionIntegrationsService.prototype, "getIntegrationsInRoom", null);
__decorate([
    typescript_rest_1.GET,
    typescript_rest_1.Path("room/:roomId/integrations/:category/:type"),
    typescript_rest_1.Security(MatrixSecurity_1.ROLE_USER),
    __param(0, typescript_rest_1.PathParam("roomId")), __param(1, typescript_rest_1.PathParam("category")), __param(2, typescript_rest_1.PathParam("type")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String, String]),
    __metadata("design:returntype", Promise)
], DimensionIntegrationsService.prototype, "getIntegrationInRoom", null);
__decorate([
    typescript_rest_1.POST,
    typescript_rest_1.Path("room/:roomId/integrations/:category/:type/config"),
    typescript_rest_1.Security(MatrixSecurity_1.ROLE_USER),
    __param(0, typescript_rest_1.PathParam("roomId")), __param(1, typescript_rest_1.PathParam("category")), __param(2, typescript_rest_1.PathParam("type")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String, String, Object]),
    __metadata("design:returntype", Promise)
], DimensionIntegrationsService.prototype, "setIntegrationConfigurationInRoom", null);
__decorate([
    typescript_rest_1.DELETE,
    typescript_rest_1.Path("room/:roomId/integrations/:category/:type"),
    typescript_rest_1.Security(MatrixSecurity_1.ROLE_USER),
    __param(0, typescript_rest_1.PathParam("roomId")), __param(1, typescript_rest_1.PathParam("category")), __param(2, typescript_rest_1.PathParam("type")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String, String]),
    __metadata("design:returntype", Promise)
], DimensionIntegrationsService.prototype, "removeIntegrationInRoom", null);
__decorate([
    typescript_rest_1.GET,
    typescript_rest_1.Path(":category/:type"),
    __param(0, typescript_rest_1.PathParam("category")), __param(1, typescript_rest_1.PathParam("type")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String]),
    __metadata("design:returntype", Promise)
], DimensionIntegrationsService.prototype, "getIntegration", null);
DimensionIntegrationsService = DimensionIntegrationsService_1 = __decorate([
    typescript_rest_1.Path("/api/v1/dimension/integrations")
], DimensionIntegrationsService);
exports.DimensionIntegrationsService = DimensionIntegrationsService;
//# sourceMappingURL=DimensionIntegrationsService.js.map