"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const typescript_rest_1 = require("typescript-rest");
const matrix_js_snippets_1 = require("matrix-js-snippets");
const IrcBridge_1 = require("../../bridges/IrcBridge");
const IrcBridgeRecord_1 = require("../../db/models/IrcBridgeRecord");
const ApiError_1 = require("../ApiError");
const MatrixSecurity_1 = require("../security/MatrixSecurity");
/**
 * API for interacting with the IRC bridge
 */
let DimensionIrcService = class DimensionIrcService {
    getOps(networkId, channelNoHash) {
        return __awaiter(this, void 0, void 0, function* () {
            const userId = this.context.request.user.userId;
            const parsed = IrcBridge_1.IrcBridge.parseNetworkId(networkId);
            const bridge = yield IrcBridgeRecord_1.default.findByPk(parsed.bridgeId);
            if (!bridge)
                throw new ApiError_1.ApiError(404, "Bridge not found");
            const client = new IrcBridge_1.IrcBridge(userId);
            const operators = yield client.getOperators(bridge, parsed.bridgeNetworkId, "#" + channelNoHash);
            matrix_js_snippets_1.LogService.info("DimensionIrcService", userId + " listed the operators for #" + channelNoHash + " on " + networkId);
            return operators;
        });
    }
    requestLink(networkId, channelNoHash, roomId, request) {
        return __awaiter(this, void 0, void 0, function* () {
            const userId = this.context.request.user.userId;
            const parsed = IrcBridge_1.IrcBridge.parseNetworkId(networkId);
            const bridge = yield IrcBridgeRecord_1.default.findByPk(parsed.bridgeId);
            if (!bridge)
                throw new ApiError_1.ApiError(404, "Bridge not found");
            const client = new IrcBridge_1.IrcBridge(userId);
            yield client.requestLink(bridge, parsed.bridgeNetworkId, "#" + channelNoHash, request.op, roomId);
            matrix_js_snippets_1.LogService.info("DimensionIrcService", userId + " requested #" + channelNoHash + " on " + networkId + " to be linked to " + roomId);
            return {}; // 200 OK
        });
    }
    unlink(networkId, channelNoHash, roomId) {
        return __awaiter(this, void 0, void 0, function* () {
            const userId = this.context.request.user.userId;
            const parsed = IrcBridge_1.IrcBridge.parseNetworkId(networkId);
            const bridge = yield IrcBridgeRecord_1.default.findByPk(parsed.bridgeId);
            if (!bridge)
                throw new ApiError_1.ApiError(404, "Bridge not found");
            const client = new IrcBridge_1.IrcBridge(userId);
            yield client.removeLink(bridge, parsed.bridgeNetworkId, "#" + channelNoHash, roomId);
            matrix_js_snippets_1.LogService.info("DimensionIrcService", userId + " unlinked #" + channelNoHash + " on " + networkId + " from " + roomId);
            return {}; // 200 OK
        });
    }
};
__decorate([
    typescript_rest_1.Context,
    __metadata("design:type", typescript_rest_1.ServiceContext)
], DimensionIrcService.prototype, "context", void 0);
__decorate([
    typescript_rest_1.GET,
    typescript_rest_1.Path(":networkId/channel/:channel/ops"),
    typescript_rest_1.Security(MatrixSecurity_1.ROLE_USER),
    __param(0, typescript_rest_1.PathParam("networkId")), __param(1, typescript_rest_1.PathParam("channel")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String]),
    __metadata("design:returntype", Promise)
], DimensionIrcService.prototype, "getOps", null);
__decorate([
    typescript_rest_1.POST,
    typescript_rest_1.Path(":networkId/channel/:channel/link/:roomId"),
    typescript_rest_1.Security(MatrixSecurity_1.ROLE_USER),
    __param(0, typescript_rest_1.PathParam("networkId")), __param(1, typescript_rest_1.PathParam("channel")), __param(2, typescript_rest_1.PathParam("roomId")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String, String, Object]),
    __metadata("design:returntype", Promise)
], DimensionIrcService.prototype, "requestLink", null);
__decorate([
    typescript_rest_1.POST,
    typescript_rest_1.Path(":networkId/channel/:channel/unlink/:roomId"),
    typescript_rest_1.Security(MatrixSecurity_1.ROLE_USER),
    __param(0, typescript_rest_1.PathParam("networkId")), __param(1, typescript_rest_1.PathParam("channel")), __param(2, typescript_rest_1.PathParam("roomId")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String, String]),
    __metadata("design:returntype", Promise)
], DimensionIrcService.prototype, "unlink", null);
DimensionIrcService = __decorate([
    typescript_rest_1.Path("/api/v1/dimension/irc")
], DimensionIrcService);
exports.DimensionIrcService = DimensionIrcService;
//# sourceMappingURL=DimensionIrcService.js.map