"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var DimensionStickerService_1;
Object.defineProperty(exports, "__esModule", { value: true });
const typescript_rest_1 = require("typescript-rest");
const MemoryCache_1 = require("../../MemoryCache");
const StickerPack_1 = require("../../db/models/StickerPack");
const Sticker_1 = require("../../db/models/Sticker");
const UserStickerPack_1 = require("../../db/models/UserStickerPack");
const ApiError_1 = require("../ApiError");
const StickerpackMetadataDownloader_1 = require("../../utils/StickerpackMetadataDownloader");
const MatrixStickerBot_1 = require("../../matrix/MatrixStickerBot");
const config_1 = require("../../config");
const MatrixSecurity_1 = require("../security/MatrixSecurity");
/**
 * API for stickers
 */
let DimensionStickerService = DimensionStickerService_1 = class DimensionStickerService {
    static getStickerPacks(enabledOnly = false) {
        return __awaiter(this, void 0, void 0, function* () {
            const cachedPacks = MemoryCache_1.Cache.for(MemoryCache_1.CACHE_STICKERS).get("packs");
            if (cachedPacks) {
                if (enabledOnly)
                    return cachedPacks.filter(p => p.isEnabled);
                return cachedPacks;
            }
            const dbPacks = yield StickerPack_1.default.findAll();
            const packs = [];
            for (const pack of dbPacks) {
                packs.push(yield DimensionStickerService_1.packToMemory(pack));
            }
            MemoryCache_1.Cache.for(MemoryCache_1.CACHE_STICKERS).put("packs", packs);
            if (enabledOnly)
                return packs.filter(p => p.isEnabled);
            return packs;
        });
    }
    getConfig() {
        return __awaiter(this, void 0, void 0, function* () {
            return {
                enabled: config_1.default.stickers.enabled,
                stickerBot: config_1.default.stickers.stickerBot,
                managerUrl: config_1.default.stickers.managerUrl,
            };
        });
    }
    getStickerPacks() {
        return __awaiter(this, void 0, void 0, function* () {
            const userId = this.context.request.user.userId;
            const cachedPacks = MemoryCache_1.Cache.for(MemoryCache_1.CACHE_STICKERS).get("packs_" + userId);
            if (cachedPacks)
                return cachedPacks;
            const allPacks = yield DimensionStickerService_1.getStickerPacks(true);
            if (allPacks.length === 0)
                return []; // We can just skip the database call
            const userPacks = yield UserStickerPack_1.default.findAll({ where: { userId: userId, isSelected: true } });
            const packs = [];
            for (const pack of allPacks) {
                const userPack = userPacks.find(p => p.packId === pack.id);
                const selectedPack = JSON.parse(JSON.stringify(pack));
                selectedPack.isSelected = userPack ? userPack.isSelected : false;
                if (!selectedPack.isSelected && pack.trackingRoomAlias)
                    continue;
                packs.push(selectedPack);
            }
            MemoryCache_1.Cache.for(MemoryCache_1.CACHE_STICKERS).put("packs_" + userId, packs);
            return packs;
        });
    }
    setPackSelected(packId, request) {
        return __awaiter(this, void 0, void 0, function* () {
            const userId = this.context.request.user.userId;
            const pack = yield StickerPack_1.default.findByPk(packId);
            if (!pack)
                throw new ApiError_1.ApiError(404, "Sticker pack not found");
            let userPack = yield UserStickerPack_1.default.findOne({ where: { userId: userId, packId: packId } });
            if (!userPack) {
                userPack = yield UserStickerPack_1.default.create({
                    packId: packId,
                    userId: userId,
                    isSelected: false,
                });
            }
            userPack.isSelected = request.isSelected;
            yield userPack.save();
            MemoryCache_1.Cache.for(MemoryCache_1.CACHE_STICKERS).del("packs_" + userId);
            return {}; // 200 OK
        });
    }
    importPack(request) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!config_1.default.stickers.enabled) {
                throw new ApiError_1.ApiError(400, "Custom stickerpacks are disabled on this homeserver");
            }
            const packUrl = request.packUrl.endsWith(".json") ? request.packUrl : `${request.packUrl}.json`;
            const metadata = yield StickerpackMetadataDownloader_1.StickerpackMetadataDownloader.getMetadata(packUrl);
            yield MatrixStickerBot_1.MatrixStickerBot.trackStickerpack(metadata.roomAlias);
            const stickerPacks = yield StickerPack_1.default.findAll({ where: { trackingRoomAlias: metadata.roomAlias } });
            MemoryCache_1.Cache.for(MemoryCache_1.CACHE_STICKERS).clear();
            if (stickerPacks.length <= 0)
                throw new ApiError_1.ApiError(500, "Stickerpack not imported");
            const pack = stickerPacks[0];
            // Simulate a call to setPackSelected
            yield this.setPackSelected(pack.id, { isSelected: true });
            const memoryPack = yield DimensionStickerService_1.packToMemory(pack);
            return Object.assign({ isSelected: true }, memoryPack);
        });
    }
    static packToMemory(pack) {
        return __awaiter(this, void 0, void 0, function* () {
            const stickers = yield Sticker_1.default.findAll({ where: { packId: pack.id } });
            return {
                id: pack.id,
                displayName: pack.name,
                avatarUrl: pack.avatarUrl,
                description: pack.description,
                isEnabled: pack.isEnabled,
                author: {
                    type: pack.authorType,
                    name: pack.authorName,
                    reference: pack.authorReference,
                },
                license: {
                    name: pack.license,
                    urlPath: pack.licensePath,
                },
                stickers: stickers.map(s => {
                    return {
                        id: s.id,
                        name: s.name,
                        description: s.description,
                        image: {
                            mxc: s.imageMxc,
                            mimetype: s.mimetype,
                        },
                        thumbnail: {
                            mxc: s.thumbnailMxc,
                            width: s.thumbnailWidth,
                            height: s.thumbnailHeight,
                        },
                    };
                }),
                trackingRoomAlias: pack.trackingRoomAlias,
            };
        });
    }
};
__decorate([
    typescript_rest_1.Context,
    __metadata("design:type", typescript_rest_1.ServiceContext)
], DimensionStickerService.prototype, "context", void 0);
__decorate([
    typescript_rest_1.GET,
    typescript_rest_1.Path("config"),
    typescript_rest_1.Security(MatrixSecurity_1.ROLE_USER),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], DimensionStickerService.prototype, "getConfig", null);
__decorate([
    typescript_rest_1.GET,
    typescript_rest_1.Path("packs"),
    typescript_rest_1.Security(MatrixSecurity_1.ROLE_USER),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], DimensionStickerService.prototype, "getStickerPacks", null);
__decorate([
    typescript_rest_1.POST,
    typescript_rest_1.Path("packs/:packId/selected"),
    typescript_rest_1.Security(MatrixSecurity_1.ROLE_USER),
    __param(0, typescript_rest_1.PathParam("packId")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Number, Object]),
    __metadata("design:returntype", Promise)
], DimensionStickerService.prototype, "setPackSelected", null);
__decorate([
    typescript_rest_1.POST,
    typescript_rest_1.Path("packs/import"),
    typescript_rest_1.Security(MatrixSecurity_1.ROLE_USER),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], DimensionStickerService.prototype, "importPack", null);
DimensionStickerService = DimensionStickerService_1 = __decorate([
    typescript_rest_1.Path("/api/v1/dimension/stickers")
], DimensionStickerService);
exports.DimensionStickerService = DimensionStickerService;
//# sourceMappingURL=DimensionStickerService.js.map