"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const typescript_rest_1 = require("typescript-rest");
const TelegramBridge_1 = require("../../bridges/TelegramBridge");
const ApiError_1 = require("../ApiError");
const MatrixSecurity_1 = require("../security/MatrixSecurity");
/**
 * API for interacting with the Telegram bridge
 */
let DimensionTelegramService = class DimensionTelegramService {
    getPortalInfo(chatId, roomId) {
        return __awaiter(this, void 0, void 0, function* () {
            const userId = this.context.request.user.userId;
            try {
                const telegram = new TelegramBridge_1.TelegramBridge(userId);
                const conf = yield telegram.getChatConfiguration(chatId, roomId);
                return {
                    bridged: conf ? conf.bridged : false,
                    canUnbridge: conf ? conf.canUnbridge : false,
                    chatId: conf ? conf.chatId : null,
                    roomId: conf ? conf.roomId : null,
                    chatName: conf ? conf.chatName : null,
                };
            }
            catch (e) {
                if (e.errcode) {
                    throw new ApiError_1.ApiError(400, "Error bridging room", e.errcode.toUpperCase());
                }
                else
                    throw e;
            }
        });
    }
    bridgeRoom(chatId, roomId, request) {
        return __awaiter(this, void 0, void 0, function* () {
            const userId = this.context.request.user.userId;
            try {
                const telegram = new TelegramBridge_1.TelegramBridge(userId);
                const portal = yield telegram.bridgeRoom(chatId, roomId, request.unbridgeOtherPortals);
                return {
                    bridged: true,
                    canUnbridge: portal.canUnbridge,
                    chatId: portal.chatId,
                    roomId: portal.roomId,
                    chatName: portal.chatName,
                };
            }
            catch (e) {
                if (e.errcode) {
                    throw new ApiError_1.ApiError(400, "Error bridging room", e.errcode.toUpperCase());
                }
                else
                    throw e;
            }
        });
    }
    unbridgeRoom(roomId) {
        return __awaiter(this, void 0, void 0, function* () {
            const userId = this.context.request.user.userId;
            try {
                const telegram = new TelegramBridge_1.TelegramBridge(userId);
                const portal = yield telegram.unbridgeRoom(roomId);
                return {
                    bridged: false,
                    canUnbridge: portal.canUnbridge,
                    chatId: portal.chatId,
                    roomId: portal.roomId,
                    chatName: portal.chatName,
                };
            }
            catch (e) {
                if (e.errcode) {
                    throw new ApiError_1.ApiError(400, "Error bridging room", e.errcode.toUpperCase());
                }
                else
                    throw e;
            }
        });
    }
};
__decorate([
    typescript_rest_1.Context,
    __metadata("design:type", typescript_rest_1.ServiceContext)
], DimensionTelegramService.prototype, "context", void 0);
__decorate([
    typescript_rest_1.GET,
    typescript_rest_1.Path("chat/:chatId"),
    typescript_rest_1.Security(MatrixSecurity_1.ROLE_USER),
    __param(0, typescript_rest_1.PathParam("chatId")), __param(1, typescript_rest_1.QueryParam("roomId")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Number, String]),
    __metadata("design:returntype", Promise)
], DimensionTelegramService.prototype, "getPortalInfo", null);
__decorate([
    typescript_rest_1.POST,
    typescript_rest_1.Path("chat/:chatId/room/:roomId"),
    typescript_rest_1.Security(MatrixSecurity_1.ROLE_USER),
    __param(0, typescript_rest_1.PathParam("chatId")), __param(1, typescript_rest_1.PathParam("roomId")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Number, String, Object]),
    __metadata("design:returntype", Promise)
], DimensionTelegramService.prototype, "bridgeRoom", null);
__decorate([
    typescript_rest_1.DELETE,
    typescript_rest_1.Path("room/:roomId"),
    typescript_rest_1.Security(MatrixSecurity_1.ROLE_USER),
    __param(0, typescript_rest_1.PathParam("roomId")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", Promise)
], DimensionTelegramService.prototype, "unbridgeRoom", null);
DimensionTelegramService = __decorate([
    typescript_rest_1.Path("/api/v1/dimension/telegram")
], DimensionTelegramService);
exports.DimensionTelegramService = DimensionTelegramService;
//# sourceMappingURL=DimensionTelegramService.js.map