"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const typescript_rest_1 = require("typescript-rest");
const WebhooksBridge_1 = require("../../bridges/WebhooksBridge");
const Webhook_1 = require("../../db/models/Webhook");
const ApiError_1 = require("../ApiError");
const matrix_js_snippets_1 = require("matrix-js-snippets");
const request = require("request");
const MatrixSecurity_1 = require("../security/MatrixSecurity");
/**
 * API for interacting with the Webhooks bridge, and for setting up proxies to other
 * services.
 */
let DimensionWebhooksService = class DimensionWebhooksService {
    postTravisCiWebhook(webhookId, payload, signature) {
        return __awaiter(this, void 0, void 0, function* () {
            const webhook = yield Webhook_1.default.findByPk(webhookId).catch(() => null);
            if (!webhook)
                throw new ApiError_1.ApiError(404, "Webhook not found");
            if (!webhook.targetUrl)
                throw new ApiError_1.ApiError(400, "Webhook not configured");
            return new Promise((resolve, _reject) => {
                request({
                    method: "POST",
                    url: webhook.targetUrl,
                    form: { payload: payload },
                    headers: {
                        "Signature": signature,
                    },
                }, (err, res, _body) => {
                    if (err) {
                        matrix_js_snippets_1.LogService.error("DimensionWebhooksService", "Error invoking travis-ci webhook");
                        matrix_js_snippets_1.LogService.error("DimensionWebhooksService", res.body);
                        throw new ApiError_1.ApiError(500, "Internal Server Error");
                    }
                    else
                        resolve();
                });
            });
        });
    }
    newWebhook(roomId, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const userId = this.context.request.user.userId;
            const webhooks = new WebhooksBridge_1.WebhooksBridge(userId);
            return webhooks.createWebhook(roomId, options);
        });
    }
    updateWebhook(roomId, hookId, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const userId = this.context.request.user.userId;
            const webhooks = new WebhooksBridge_1.WebhooksBridge(userId);
            return webhooks.updateWebhook(roomId, hookId, options);
        });
    }
    deleteWebhook(roomId, hookId) {
        return __awaiter(this, void 0, void 0, function* () {
            const userId = this.context.request.user.userId;
            const webhooks = new WebhooksBridge_1.WebhooksBridge(userId);
            return webhooks.deleteWebhook(roomId, hookId);
        });
    }
};
__decorate([
    typescript_rest_1.Context,
    __metadata("design:type", typescript_rest_1.ServiceContext)
], DimensionWebhooksService.prototype, "context", void 0);
__decorate([
    typescript_rest_1.POST,
    typescript_rest_1.Path("/travisci/:webhookId"),
    __param(0, typescript_rest_1.PathParam("webhookId")), __param(1, typescript_rest_1.FormParam("payload")), __param(2, typescript_rest_1.HeaderParam("Signature")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String, String]),
    __metadata("design:returntype", Promise)
], DimensionWebhooksService.prototype, "postTravisCiWebhook", null);
__decorate([
    typescript_rest_1.POST,
    typescript_rest_1.Path("room/:roomId/webhooks/new"),
    typescript_rest_1.Security(MatrixSecurity_1.ROLE_USER),
    __param(0, typescript_rest_1.PathParam("roomId")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, Object]),
    __metadata("design:returntype", Promise)
], DimensionWebhooksService.prototype, "newWebhook", null);
__decorate([
    typescript_rest_1.POST,
    typescript_rest_1.Path("room/:roomId/webhooks/:hookId"),
    typescript_rest_1.Security(MatrixSecurity_1.ROLE_USER),
    __param(0, typescript_rest_1.PathParam("roomId")), __param(1, typescript_rest_1.PathParam("hookId")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String, Object]),
    __metadata("design:returntype", Promise)
], DimensionWebhooksService.prototype, "updateWebhook", null);
__decorate([
    typescript_rest_1.DELETE,
    typescript_rest_1.Path("room/:roomId/webhooks/:hookId"),
    typescript_rest_1.Security(MatrixSecurity_1.ROLE_USER),
    __param(0, typescript_rest_1.PathParam("roomId")), __param(1, typescript_rest_1.PathParam("hookId")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String]),
    __metadata("design:returntype", Promise)
], DimensionWebhooksService.prototype, "deleteWebhook", null);
DimensionWebhooksService = __decorate([
    typescript_rest_1.Path("/api/v1/dimension/webhooks")
], DimensionWebhooksService);
exports.DimensionWebhooksService = DimensionWebhooksService;
//# sourceMappingURL=DimensionWebhooksService.js.map