"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const typescript_rest_1 = require("typescript-rest");
const matrix_js_snippets_1 = require("matrix-js-snippets");
const url = require("url");
const ApiError_1 = require("../ApiError");
const dns = require("dns-then");
const config_1 = require("../../config");
const netmask_1 = require("netmask");
const request = require("request");
const TermsController_1 = require("../controllers/TermsController");
const TermsRecord_1 = require("../../db/models/TermsRecord");
const TermsTextRecord_1 = require("../../db/models/TermsTextRecord");
/**
 * API for widgets
 */
let DimensionWidgetService = class DimensionWidgetService {
    getPolicy(shortcode, language, version) {
        return __awaiter(this, void 0, void 0, function* () {
            if (version === TermsController_1.VERSION_DRAFT) {
                throw new ApiError_1.ApiError(401, "Cannot access draft versions of policies", "M_UNAUTHORIZED");
            }
            const terms = yield TermsRecord_1.default.findOne({ where: { shortcode, version }, include: [TermsTextRecord_1.default] });
            if (!terms)
                throw new ApiError_1.ApiError(404, "Not found", "M_NOT_FOUND");
            const text = terms.texts.find(t => t.language === language);
            return {
                name: text.name,
                text: text.text,
            };
        });
    }
    isEmbeddable(checkUrl) {
        return __awaiter(this, void 0, void 0, function* () {
            matrix_js_snippets_1.LogService.info("DimensionWidgetService", "Checking to see if a url is embeddable: " + checkUrl);
            const parsed = url.parse(checkUrl);
            // Only allow http and https
            if (parsed.protocol !== "http:" && parsed.protocol !== "https:") {
                throw new ApiError_1.ApiError(400, "Invalid scheme: " + parsed.protocol);
            }
            // Get the IP address we're trying to connect to so we can ensure it's not blacklisted
            const hostname = parsed.hostname.split(":")[0];
            let addresses = [];
            try {
                addresses = yield dns.resolve(hostname);
            }
            catch (err) {
                matrix_js_snippets_1.LogService.error("DimensionWidgetService", err);
            }
            if (!addresses || addresses.length === 0)
                throw new ApiError_1.ApiError(400, "Cannot resolve host " + hostname);
            // Check the blacklist
            for (const ipOrCidr of config_1.default.widgetBlacklist) {
                const block = new netmask_1.Netmask(ipOrCidr);
                for (const address of addresses) {
                    if (block.contains(address)) {
                        throw new ApiError_1.ApiError(400, "Address blacklisted");
                    }
                }
            }
            // Now we need to verify we can actually make the request
            yield new Promise((resolve, reject) => {
                request(checkUrl, (err, response) => {
                    if (err) {
                        matrix_js_snippets_1.LogService.error("DimensionWidgetService", err);
                        return reject(new ApiError_1.ApiError(400, "Failed to contact host"));
                    }
                    if (response.statusCode >= 200 && response.statusCode < 300) {
                        // 200 OK
                        const xFrameOptions = (response.headers["x-frame-options"] || '').toLowerCase();
                        if (xFrameOptions === "sameorigin" || xFrameOptions === "deny") {
                            return reject(new ApiError_1.ApiError(400, "X-Frame-Options prevents embedding"));
                        }
                        resolve();
                    }
                    else {
                        matrix_js_snippets_1.LogService.verbose("DimensionWidgetService", response.body);
                        return reject(new ApiError_1.ApiError(400, "Non-success status code returned"));
                    }
                });
            });
            return { canEmbed: true };
        });
    }
};
__decorate([
    typescript_rest_1.GET,
    typescript_rest_1.Path("/terms/:shortcode/:language/:version"),
    __param(0, typescript_rest_1.PathParam("shortcode")), __param(1, typescript_rest_1.PathParam("language")), __param(2, typescript_rest_1.PathParam("version")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String, String]),
    __metadata("design:returntype", Promise)
], DimensionWidgetService.prototype, "getPolicy", null);
__decorate([
    typescript_rest_1.GET,
    typescript_rest_1.Path("embeddable"),
    __param(0, typescript_rest_1.QueryParam("url")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", Promise)
], DimensionWidgetService.prototype, "isEmbeddable", null);
DimensionWidgetService = __decorate([
    typescript_rest_1.Path("/api/v1/dimension/widgets")
], DimensionWidgetService);
exports.DimensionWidgetService = DimensionWidgetService;
//# sourceMappingURL=DimensionWidgetService.js.map