"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const typescript_rest_1 = require("typescript-rest");
const ApiError_1 = require("../ApiError");
const matrix_js_snippets_1 = require("matrix-js-snippets");
const AppserviceStore_1 = require("../../db/AppserviceStore");
/**
 * API for handling appservice traffic from a homeserver
 */
let MatrixAppServiceApiService = class MatrixAppServiceApiService {
    onTransaction(homeserverToken, txnId, _txn) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const appservice = yield AppserviceStore_1.AppserviceStore.getByHomeserverToken(homeserverToken);
                // We don't handle the transaction at all - we just don't want the homeserver to consider us down
                matrix_js_snippets_1.LogService.verbose("MatrixAppServiceApiService", "Accepting transaction " + txnId + " for appservice " + appservice.id + " blindly");
                return {}; // 200 OK
            }
            catch (err) {
                matrix_js_snippets_1.LogService.error("MatrixAppServiceApiService", err);
                throw new ApiError_1.ApiError(403, { errcode: "M_FORBIDDEN" });
            }
        });
    }
    getRoom(homeserverToken, roomAlias) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const appservice = yield AppserviceStore_1.AppserviceStore.getByHomeserverToken(homeserverToken);
                // We don't support room lookups
                matrix_js_snippets_1.LogService.verbose("MatrixAppServiceApiService", "404ing request for room " + roomAlias + " at appservice " + appservice.id);
                throw new ApiError_1.ApiError(404, { errcode: "IO.T2BOT.DIMENSION.ROOMS_NOT_SUPPORTED" });
            }
            catch (err) {
                if (err instanceof ApiError_1.ApiError)
                    throw err;
                matrix_js_snippets_1.LogService.error("MatrixAppServiceApiService", err);
                throw new ApiError_1.ApiError(403, { errcode: "M_FORBIDDEN" });
            }
        });
    }
    getUser(homeserverToken, userId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const appservice = yield AppserviceStore_1.AppserviceStore.getByHomeserverToken(homeserverToken);
                try {
                    const user = yield AppserviceStore_1.AppserviceStore.getUser(appservice.id, userId);
                    return {
                        userId: user.id,
                        displayName: user.displayName,
                        avatarUrl: user.avatarUrl,
                    };
                }
                catch (err) {
                    matrix_js_snippets_1.LogService.error("MatrixAppServiceApiService", err);
                    throw new ApiError_1.ApiError(404, { errcode: "IO.T2BOT.DIMENSION.USER_NOT_FOUND" });
                }
            }
            catch (err) {
                if (err instanceof ApiError_1.ApiError)
                    throw err;
                matrix_js_snippets_1.LogService.error("MatrixAppServiceApiService", err);
                throw new ApiError_1.ApiError(403, { errcode: "M_FORBIDDEN" });
            }
        });
    }
};
__decorate([
    typescript_rest_1.PUT,
    typescript_rest_1.Path("/transactions/:txnId"),
    __param(0, typescript_rest_1.QueryParam("access_token")), __param(1, typescript_rest_1.PathParam("txnId")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String, Object]),
    __metadata("design:returntype", Promise)
], MatrixAppServiceApiService.prototype, "onTransaction", null);
__decorate([
    typescript_rest_1.GET,
    typescript_rest_1.Path("/room/:alias"),
    __param(0, typescript_rest_1.QueryParam("access_token")), __param(1, typescript_rest_1.PathParam("alias")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String]),
    __metadata("design:returntype", Promise)
], MatrixAppServiceApiService.prototype, "getRoom", null);
__decorate([
    typescript_rest_1.GET,
    typescript_rest_1.Path("/user/:userId"),
    __param(0, typescript_rest_1.QueryParam("access_token")), __param(1, typescript_rest_1.PathParam("userId")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String]),
    __metadata("design:returntype", Promise)
], MatrixAppServiceApiService.prototype, "getUser", null);
MatrixAppServiceApiService = __decorate([
    typescript_rest_1.Path("/_matrix/appservice/r0"),
    typescript_rest_1.Path("/_matrix/app/v1") // the matrix spec version
], MatrixAppServiceApiService);
exports.MatrixAppServiceApiService = MatrixAppServiceApiService;
//# sourceMappingURL=MatrixAppServiceApiService.js.map