"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const typescript_rest_1 = require("typescript-rest");
const ApiError_1 = require("../ApiError");
const es6_1 = require("typescript-ioc/es6");
const AccountController_1 = require("../controllers/AccountController");
const MatrixSecurity_1 = require("../security/MatrixSecurity");
const TermsController_1 = require("../controllers/TermsController");
const ScalarClient_1 = require("../../scalar/ScalarClient");
/**
 * API for the minimum Scalar API we need to implement to be compatible with clients. Used for registration
 * and general account management.
 */
let ScalarService = class ScalarService {
    register(request, apiVersion) {
        return __awaiter(this, void 0, void 0, function* () {
            if (apiVersion !== "1.1") {
                throw new ApiError_1.ApiError(401, "Invalid API version.");
            }
            const response = yield this.accountController.registerAccount(request, ScalarClient_1.ScalarClient.KIND_LEGACY);
            return { scalar_token: response.token };
        });
    }
    getAccount(apiVersion) {
        return __awaiter(this, void 0, void 0, function* () {
            if (apiVersion !== "1.1") {
                throw new ApiError_1.ApiError(401, "Invalid API version.");
            }
            return { user_id: this.context.request.user.userId };
        });
    }
    getTerms() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.termsController.getAvailableTerms();
        });
    }
    signTerms(request) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.termsController.signTermsMatching(this.context.request.user, request.user_accepts);
            return {};
        });
    }
    ping() {
        return __awaiter(this, void 0, void 0, function* () {
            return {}; // 200 OK
        });
    }
};
__decorate([
    es6_1.Inject,
    __metadata("design:type", AccountController_1.default)
], ScalarService.prototype, "accountController", void 0);
__decorate([
    es6_1.Inject,
    __metadata("design:type", TermsController_1.default)
], ScalarService.prototype, "termsController", void 0);
__decorate([
    typescript_rest_1.Context,
    __metadata("design:type", typescript_rest_1.ServiceContext)
], ScalarService.prototype, "context", void 0);
__decorate([
    typescript_rest_1.POST,
    typescript_rest_1.Path("register"),
    __param(1, typescript_rest_1.QueryParam("v")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], ScalarService.prototype, "register", null);
__decorate([
    typescript_rest_1.GET,
    typescript_rest_1.Path("account"),
    typescript_rest_1.Security(MatrixSecurity_1.ROLE_USER),
    __param(0, typescript_rest_1.QueryParam("v")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", Promise)
], ScalarService.prototype, "getAccount", null);
__decorate([
    typescript_rest_1.GET,
    typescript_rest_1.Path("terms"),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], ScalarService.prototype, "getTerms", null);
__decorate([
    typescript_rest_1.POST,
    typescript_rest_1.Path("terms"),
    typescript_rest_1.Security(MatrixSecurity_1.ROLE_USER),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], ScalarService.prototype, "signTerms", null);
__decorate([
    typescript_rest_1.GET,
    typescript_rest_1.Path("ping"),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], ScalarService.prototype, "ping", null);
ScalarService = __decorate([
    typescript_rest_1.Path("/api/v1/scalar"),
    es6_1.AutoWired
], ScalarService);
exports.ScalarService = ScalarService;
//# sourceMappingURL=ScalarService.js.map