"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Upstream_1 = require("../db/models/Upstream");
const UserScalarToken_1 = require("../db/models/UserScalarToken");
const matrix_js_snippets_1 = require("matrix-js-snippets");
const request = require("request");
const SlackBridgeRecord_1 = require("../db/models/SlackBridgeRecord");
class SlackBridge {
    constructor(requestingUserId) {
        this.requestingUserId = requestingUserId;
    }
    getDefaultBridge() {
        return __awaiter(this, void 0, void 0, function* () {
            const bridges = yield SlackBridgeRecord_1.default.findAll({ where: { isEnabled: true } });
            if (!bridges || bridges.length !== 1) {
                throw new Error("No bridges or too many bridges found");
            }
            return bridges[0];
        });
    }
    isBridgingEnabled() {
        return __awaiter(this, void 0, void 0, function* () {
            const bridges = yield SlackBridgeRecord_1.default.findAll({ where: { isEnabled: true } });
            return !!bridges && bridges.length > 0;
        });
    }
    getBridgeInfo() {
        return __awaiter(this, void 0, void 0, function* () {
            const bridge = yield this.getDefaultBridge();
            if (bridge.upstreamId) {
                const info = yield this.doUpstreamRequest(bridge, "POST", "/bridges/slack/_matrix/provision/getbotid/", null, {});
                if (!info || !info.replies || !info.replies[0] || !info.replies[0].response) {
                    throw new Error("Invalid response from Modular for Slack bot user ID");
                }
                return { botUserId: info.replies[0].response.bot_user_id };
            }
            else {
                const info = yield this.doProvisionRequest(bridge, "POST", "/_matrix/provision/getbotid");
                return { botUserId: info.bot_user_id };
            }
        });
    }
    getLink(roomId) {
        return __awaiter(this, void 0, void 0, function* () {
            const bridge = yield this.getDefaultBridge();
            const requestBody = {
                matrix_room_id: roomId,
                user_id: this.requestingUserId,
            };
            try {
                if (bridge.upstreamId) {
                    delete requestBody["user_id"];
                    const link = yield this.doUpstreamRequest(bridge, "POST", "/bridges/slack/_matrix/provision/getlink", null, requestBody);
                    if (!link || !link.replies || !link.replies[0] || !link.replies[0].response) {
                        // noinspection ExceptionCaughtLocallyJS
                        throw new Error("Invalid response from Modular for Slack list links in " + roomId);
                    }
                    return {
                        roomId: link.replies[0].response.matrix_room_id,
                        isWebhook: link.replies[0].response.isWebhook,
                        channelName: link.replies[0].response.slack_channel_name,
                        channelId: link.replies[0].response.slack_channel_id,
                        teamId: link.replies[0].response.team_id,
                    };
                }
                else {
                    const link = yield this.doProvisionRequest(bridge, "POST", "/_matrix/provision/getlink", null, requestBody);
                    return {
                        roomId: link.matrix_room_id,
                        isWebhook: link.isWebhook,
                        channelName: link.slack_channel_name,
                        channelId: link.slack_channel_id,
                        teamId: link.team_id,
                    };
                }
            }
            catch (e) {
                if (e.status === 404)
                    return null;
                matrix_js_snippets_1.LogService.error("SlackBridge", e);
                throw e;
            }
        });
    }
    requestEventsLink(roomId, teamId, channelId) {
        return __awaiter(this, void 0, void 0, function* () {
            const bridge = yield this.getDefaultBridge();
            const requestBody = {
                matrix_room_id: roomId,
                channel_id: channelId,
                team_id: teamId,
                user_id: this.requestingUserId,
            };
            if (bridge.upstreamId) {
                delete requestBody["user_id"];
                yield this.doUpstreamRequest(bridge, "POST", "/bridges/slack/_matrix/provision/link", null, requestBody);
            }
            else {
                yield this.doProvisionRequest(bridge, "POST", "/_matrix/provision/link", null, requestBody);
            }
        });
    }
    removeEventsLink(roomId, teamId, channelId) {
        return __awaiter(this, void 0, void 0, function* () {
            const bridge = yield this.getDefaultBridge();
            const requestBody = {
                matrix_room_id: roomId,
                channel_id: channelId,
                team_id: teamId,
                user_id: this.requestingUserId,
            };
            if (bridge.upstreamId) {
                delete requestBody["user_id"];
                yield this.doUpstreamRequest(bridge, "POST", "/bridges/slack/_matrix/provision/unlink", null, requestBody);
            }
            else {
                yield this.doProvisionRequest(bridge, "POST", "/_matrix/provision/unlink", null, requestBody);
            }
        });
    }
    removeWebhooksLink(roomId) {
        return __awaiter(this, void 0, void 0, function* () {
            const bridge = yield this.getDefaultBridge();
            const requestBody = {
                matrix_room_id: roomId,
                user_id: this.requestingUserId,
            };
            if (bridge.upstreamId) {
                delete requestBody["user_id"];
                yield this.doUpstreamRequest(bridge, "POST", "/bridges/slack/_matrix/provision/unlink", null, requestBody);
            }
            else {
                yield this.doProvisionRequest(bridge, "POST", "/_matrix/provision/unlink", null, requestBody);
            }
        });
    }
    getChannels(teamId) {
        return __awaiter(this, void 0, void 0, function* () {
            const bridge = yield this.getDefaultBridge();
            const requestBody = {
                team_id: teamId,
                user_id: this.requestingUserId,
            };
            try {
                if (bridge.upstreamId) {
                    delete requestBody["user_id"];
                    const response = yield this.doUpstreamRequest(bridge, "POST", "/bridges/slack/_matrix/provision/channels", null, requestBody);
                    if (!response || !response.replies || !response.replies[0] || !response.replies[0].response) {
                        // noinspection ExceptionCaughtLocallyJS
                        throw new Error("Invalid response from Modular for Slack get channels of " + teamId);
                    }
                    return response.replies[0].response.channels;
                }
                else {
                    const response = yield this.doProvisionRequest(bridge, "POST", "/_matrix/provision/channels", null, requestBody);
                    return response.channels;
                }
            }
            catch (e) {
                if (e.status === 404)
                    return null;
                matrix_js_snippets_1.LogService.error("SlackBridge", e);
                throw e;
            }
        });
    }
    getTeams() {
        return __awaiter(this, void 0, void 0, function* () {
            const bridge = yield this.getDefaultBridge();
            const requestBody = {
                user_id: this.requestingUserId,
            };
            try {
                if (bridge.upstreamId) {
                    delete requestBody["user_id"];
                    const response = yield this.doUpstreamRequest(bridge, "POST", "/bridges/slack/_matrix/provision/teams", null, requestBody);
                    if (!response || !response.replies || !response.replies[0] || !response.replies[0].response) {
                        // noinspection ExceptionCaughtLocallyJS
                        throw new Error("Invalid response from Modular for Slack get teams for " + this.requestingUserId);
                    }
                    return response.replies[0].response.teams;
                }
                else {
                    const response = yield this.doProvisionRequest(bridge, "POST", "/_matrix/provision/teams", null, requestBody);
                    return response.teams;
                }
            }
            catch (e) {
                if (e.status === 404)
                    return null;
                matrix_js_snippets_1.LogService.error("SlackBridge", e);
                throw e;
            }
        });
    }
    getAuthUrl() {
        return __awaiter(this, void 0, void 0, function* () {
            const bridge = yield this.getDefaultBridge();
            const requestBody = {
                user_id: this.requestingUserId,
            };
            try {
                if (bridge.upstreamId) {
                    delete requestBody["user_id"];
                    const response = yield this.doUpstreamRequest(bridge, "POST", "/bridges/slack/_matrix/provision/authurl", null, requestBody);
                    if (!response || !response.replies || !response.replies[0] || !response.replies[0].response) {
                        // noinspection ExceptionCaughtLocallyJS
                        throw new Error("Invalid response from Modular for Slack get auth url for " + this.requestingUserId);
                    }
                    return response.replies[0].response.auth_uri;
                }
                else {
                    const response = yield this.doProvisionRequest(bridge, "POST", "/_matrix/provision/authurl", null, requestBody);
                    return response.auth_uri;
                }
            }
            catch (e) {
                if (e.status === 404)
                    return null;
                matrix_js_snippets_1.LogService.error("SlackBridge", e);
                throw e;
            }
        });
    }
    doUpstreamRequest(bridge, method, endpoint, qs, body) {
        return __awaiter(this, void 0, void 0, function* () {
            const upstream = yield Upstream_1.default.findByPk(bridge.upstreamId);
            const token = yield UserScalarToken_1.default.findOne({
                where: {
                    upstreamId: upstream.id,
                    isDimensionToken: false,
                    userId: this.requestingUserId,
                },
            });
            if (!qs)
                qs = {};
            qs["scalar_token"] = token.scalarToken;
            const apiUrl = upstream.apiUrl.endsWith("/") ? upstream.apiUrl.substring(0, upstream.apiUrl.length - 1) : upstream.apiUrl;
            const url = apiUrl + (endpoint.startsWith("/") ? endpoint : "/" + endpoint);
            matrix_js_snippets_1.LogService.info("SlackBridge", "Doing upstream Slack Bridge request: " + url);
            return new Promise((resolve, reject) => {
                request({
                    method: method,
                    url: url,
                    qs: qs,
                    json: body,
                }, (err, res, _body) => {
                    try {
                        if (err) {
                            matrix_js_snippets_1.LogService.error("SlackBridge", "Error calling " + url);
                            matrix_js_snippets_1.LogService.error("SlackBridge", err);
                            reject(err);
                        }
                        else if (!res) {
                            matrix_js_snippets_1.LogService.error("SlackBridge", "There is no response for " + url);
                            reject(new Error("No response provided - is the service online?"));
                        }
                        else if (res.statusCode !== 200) {
                            if (typeof (res.body) === "string")
                                res.body = JSON.parse(res.body);
                            matrix_js_snippets_1.LogService.error("SlackBridge", "Got status code " + res.statusCode + " when calling " + url);
                            matrix_js_snippets_1.LogService.error("SlackBridge", res.body);
                            reject({ body: res.body, status: res.statusCode });
                        }
                        else {
                            if (typeof (res.body) === "string")
                                res.body = JSON.parse(res.body);
                            resolve(res.body);
                        }
                    }
                    catch (e) {
                        matrix_js_snippets_1.LogService.error("SlackBridge", e);
                        reject(e);
                    }
                });
            });
        });
    }
    doProvisionRequest(bridge, method, endpoint, qs, body) {
        return __awaiter(this, void 0, void 0, function* () {
            const provisionUrl = bridge.provisionUrl;
            const apiUrl = provisionUrl.endsWith("/") ? provisionUrl.substring(0, provisionUrl.length - 1) : provisionUrl;
            const url = apiUrl + (endpoint.startsWith("/") ? endpoint : "/" + endpoint);
            matrix_js_snippets_1.LogService.info("SlackBridge", "Doing provision Slack Bridge request: " + url);
            return new Promise((resolve, reject) => {
                request({
                    method: method,
                    url: url,
                    qs: qs,
                    json: body,
                }, (err, res, _body) => {
                    try {
                        if (err) {
                            matrix_js_snippets_1.LogService.error("SlackBridge", "Error calling" + url);
                            matrix_js_snippets_1.LogService.error("SlackBridge", err);
                            reject(err);
                        }
                        else if (!res) {
                            matrix_js_snippets_1.LogService.error("SlackBridge", "There is no response for " + url);
                            reject(new Error("No response provided - is the service online?"));
                        }
                        else if (res.statusCode !== 200) {
                            if (typeof (res.body) === "string")
                                res.body = JSON.parse(res.body);
                            matrix_js_snippets_1.LogService.error("SlackBridge", "Got status code " + res.statusCode + " when calling " + url);
                            matrix_js_snippets_1.LogService.error("SlackBridge", res.body);
                            reject({ body: res.body, status: res.statusCode });
                        }
                        else {
                            if (typeof (res.body) === "string")
                                res.body = JSON.parse(res.body);
                            resolve(res.body);
                        }
                    }
                    catch (e) {
                        matrix_js_snippets_1.LogService.error("SlackBridge", e);
                        reject(e);
                    }
                });
            });
        });
    }
}
exports.SlackBridge = SlackBridge;
//# sourceMappingURL=SlackBridge.js.map