"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const matrix_js_snippets_1 = require("matrix-js-snippets");
const request = require("request");
const WebhookBridgeRecord_1 = require("../db/models/WebhookBridgeRecord");
class WebhooksBridge {
    constructor(requestingUserId) {
        this.requestingUserId = requestingUserId;
    }
    getDefaultBridge() {
        return __awaiter(this, void 0, void 0, function* () {
            const bridges = yield WebhookBridgeRecord_1.default.findAll({ where: { isEnabled: true } });
            if (!bridges || bridges.length !== 1) {
                throw new Error("No bridges or too many bridges found");
            }
            return bridges[0];
        });
    }
    isBridgingEnabled() {
        return __awaiter(this, void 0, void 0, function* () {
            const bridges = yield WebhookBridgeRecord_1.default.findAll({ where: { isEnabled: true } });
            return !!bridges && bridges.length > 0;
        });
    }
    getBridgeInfo() {
        return __awaiter(this, void 0, void 0, function* () {
            const bridge = yield this.getDefaultBridge();
            return this.doProvisionRequest(bridge, "GET", "/api/v1/provision/info");
        });
    }
    getHooks(roomId) {
        return __awaiter(this, void 0, void 0, function* () {
            const bridge = yield this.getDefaultBridge();
            try {
                const response = yield this.doProvisionRequest(bridge, "GET", `/api/v1/provision/${roomId}/hooks`);
                if (!response.success)
                    throw new Error("Failed to get webhooks");
                return response.results;
            }
            catch (e) {
                matrix_js_snippets_1.LogService.error("WebhooksBridge", e);
                return [];
            }
        });
    }
    createWebhook(roomId, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const bridge = yield this.getDefaultBridge();
            return this.doProvisionRequest(bridge, "PUT", `/api/v1/provision/${roomId}/hook`, null, options);
        });
    }
    updateWebhook(roomId, hookId, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const bridge = yield this.getDefaultBridge();
            return this.doProvisionRequest(bridge, "PUT", `/api/v1/provision/${roomId}/hook/${hookId}`, null, options);
        });
    }
    deleteWebhook(roomId, hookId) {
        return __awaiter(this, void 0, void 0, function* () {
            const bridge = yield this.getDefaultBridge();
            return this.doProvisionRequest(bridge, "DELETE", `/api/v1/provision/${roomId}/hook/${hookId}`);
        });
    }
    doProvisionRequest(bridge, method, endpoint, qs, body) {
        return __awaiter(this, void 0, void 0, function* () {
            const provisionUrl = bridge.provisionUrl;
            const apiUrl = provisionUrl.endsWith("/") ? provisionUrl.substring(0, provisionUrl.length - 1) : provisionUrl;
            const url = apiUrl + (endpoint.startsWith("/") ? endpoint : "/" + endpoint);
            matrix_js_snippets_1.LogService.info("WebhooksBridge", "Doing provision Webhooks Bridge request: " + url);
            if (!qs)
                qs = {};
            if (!qs["userId"])
                qs["userId"] = this.requestingUserId;
            qs["token"] = bridge.sharedSecret;
            return new Promise((resolve, reject) => {
                request({
                    method: method,
                    url: url,
                    qs: qs,
                    json: body,
                }, (err, res, _body) => {
                    try {
                        if (err) {
                            matrix_js_snippets_1.LogService.error("WebhooksBridge", "Error calling" + url);
                            matrix_js_snippets_1.LogService.error("WebhooksBridge", err);
                            reject(err);
                        }
                        else if (!res) {
                            matrix_js_snippets_1.LogService.error("WebhooksBridge", "There is no response for " + url);
                            reject(new Error("No response provided - is the service online?"));
                        }
                        else if (res.statusCode !== 200) {
                            matrix_js_snippets_1.LogService.error("WebhooksBridge", "Got status code " + res.statusCode + " when calling " + url);
                            matrix_js_snippets_1.LogService.error("WebhooksBridge", res.body);
                            reject(new Error("Request failed"));
                        }
                        else {
                            if (typeof (res.body) === "string")
                                res.body = JSON.parse(res.body);
                            resolve(res.body);
                        }
                    }
                    catch (e) {
                        matrix_js_snippets_1.LogService.error("WebhooksBridge", e);
                        reject(e);
                    }
                });
            });
        });
    }
}
exports.WebhooksBridge = WebhooksBridge;
//# sourceMappingURL=WebhooksBridge.js.map