"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const AppService_1 = require("./models/AppService");
const AppServiceUser_1 = require("./models/AppServiceUser");
const randomString = require("random-string");
const MatrixAppserviceClient_1 = require("../matrix/MatrixAppserviceClient");
const matrix_js_snippets_1 = require("matrix-js-snippets");
class AppserviceStore {
    static create(userPrefix) {
        return __awaiter(this, void 0, void 0, function* () {
            const id = "dimension-" + randomString({ length: 25 });
            const asToken = randomString({ length: 100 });
            const hsToken = randomString({ length: 100 });
            return AppService_1.default.create({
                id: id,
                asToken: asToken,
                hsToken: hsToken,
                userPrefix: userPrefix,
            });
        });
    }
    static getUser(appserviceId, userId) {
        return __awaiter(this, void 0, void 0, function* () {
            const user = yield AppServiceUser_1.default.findOne({ where: { appserviceId: appserviceId, id: userId } });
            if (!user)
                throw new Error("User not found");
            return user;
        });
    }
    static getByHomeserverToken(hsToken) {
        return __awaiter(this, void 0, void 0, function* () {
            const appservice = yield AppService_1.default.findOne({ where: { hsToken: hsToken } });
            if (!appservice)
                throw new Error("Appservice not found");
            return appservice;
        });
    }
    static getAllByUserPrefix(userPrefix) {
        return __awaiter(this, void 0, void 0, function* () {
            return AppService_1.default.findAll({ where: { userPrefix: userPrefix } });
        });
    }
    static getSafeUserId(userIdOrPrefix) {
        // Force user IDs to be lowercase and strip out characters that aren't permitted
        // https://matrix.org/docs/spec/appendices.html#user-identifiers
        if (userIdOrPrefix.startsWith('@')) {
            // we only change out the parts we care about. The rest will be crushed down.
            userIdOrPrefix = userIdOrPrefix.replace(/@/g, '=40');
            userIdOrPrefix = userIdOrPrefix.replace(/:/g, '=3A');
        }
        return userIdOrPrefix.toLowerCase().replace(/[^a-z0-9._\-=]/g, '.');
    }
    static registerUser(appserviceId, userId) {
        return __awaiter(this, void 0, void 0, function* () {
            const appservice = yield AppService_1.default.findOne({ where: { id: appserviceId } });
            if (!appservice)
                throw new Error("Appservice not found");
            matrix_js_snippets_1.LogService.info("AppserviceStore", "Registering to own " + userId + " in appservice " + appserviceId);
            const client = new MatrixAppserviceClient_1.MatrixAppserviceClient(appservice);
            const localpart = AppserviceStore.getSafeUserId(userId.substring(1).split(":")[0]);
            const response = yield client.registerUser(localpart);
            matrix_js_snippets_1.LogService.info("AppserviceStore", "Successfully registered " + userId);
            return yield AppServiceUser_1.default.create({
                id: userId,
                appserviceId: appserviceId,
                accessToken: response.access_token,
            });
        });
    }
    static getOrCreateUser(appserviceId, userId) {
        return __awaiter(this, void 0, void 0, function* () {
            const user = yield AppServiceUser_1.default.findOne({ where: { appserviceId: appserviceId, id: userId } });
            if (!user)
                return AppserviceStore.registerUser(appserviceId, userId);
            return user;
        });
    }
    static getAppservice(id) {
        return __awaiter(this, void 0, void 0, function* () {
            const appservice = yield AppService_1.default.findByPk(id);
            if (!appservice)
                throw new Error("Appservice not found");
            return appservice;
        });
    }
    constructor() {
    }
}
exports.AppserviceStore = AppserviceStore;
//# sourceMappingURL=AppserviceStore.js.map