"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const CustomSimpleBotRecord_1 = require("./models/CustomSimpleBotRecord");
const MemoryCache_1 = require("../MemoryCache");
const MatrixLiteClient_1 = require("../matrix/MatrixLiteClient");
const config_1 = require("../config");
const randomString = require("random-string");
const matrix_js_snippets_1 = require("matrix-js-snippets");
class BotStore {
    constructor() {
    }
    static createCustom(template) {
        return __awaiter(this, void 0, void 0, function* () {
            const newType = `${BotStore.TYPE_PREFIX}${randomString({ length: 32 })}`;
            const bot = yield CustomSimpleBotRecord_1.default.create(Object.assign({}, template, { type: newType }));
            return BotStore.mapAndCacheCustom(bot);
        });
    }
    static updateCustom(id, template) {
        return __awaiter(this, void 0, void 0, function* () {
            const record = yield CustomSimpleBotRecord_1.default.findByPk(id);
            if (!record)
                return null;
            record.name = template.name;
            record.avatarUrl = template.avatarUrl;
            record.description = template.description;
            record.isEnabled = template.isEnabled;
            record.isPublic = template.isPublic;
            record.userId = template.userId;
            record.accessToken = template.accessToken;
            yield record.save();
            return BotStore.mapAndCacheCustom(record);
        });
    }
    static deleteCustom(id) {
        return __awaiter(this, void 0, void 0, function* () {
            const record = yield CustomSimpleBotRecord_1.default.findByPk(id);
            if (!record)
                return null;
            yield record.destroy();
            MemoryCache_1.Cache.for(MemoryCache_1.CACHE_SIMPLE_BOTS).del("bots");
            return true;
        });
    }
    static removeCustomByTypeFromRoom(type, roomId) {
        return __awaiter(this, void 0, void 0, function* () {
            const record = yield CustomSimpleBotRecord_1.default.findOne({ where: { type: type } });
            if (!record)
                return null;
            try {
                const client = new MatrixLiteClient_1.MatrixLiteClient(record.accessToken);
                yield client.leaveRoom(roomId);
            }
            catch (e) {
                matrix_js_snippets_1.LogService.error("BotStore", "Error leaving room " + roomId + " for integration type " + type);
                matrix_js_snippets_1.LogService.error("BotStore", e);
                throw new Error("Error leaving room");
            }
        });
    }
    static getCustomBot(id) {
        return __awaiter(this, void 0, void 0, function* () {
            const bots = yield BotStore.getCustomBots();
            return bots.find(b => b.id === id);
        });
    }
    static getCustomBots() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!MemoryCache_1.Cache.for(MemoryCache_1.CACHE_SIMPLE_BOTS).get("bots")) {
                const bots = yield CustomSimpleBotRecord_1.default.findAll();
                const cached = [];
                yield Promise.all(bots.map((b) => __awaiter(this, void 0, void 0, function* () {
                    cached.push(yield BotStore.mapAndCacheCustom(b));
                    return Promise.resolve();
                })));
                MemoryCache_1.Cache.for(MemoryCache_1.CACHE_SIMPLE_BOTS).put("bots", cached);
            }
            return MemoryCache_1.Cache.for(MemoryCache_1.CACHE_SIMPLE_BOTS).get("bots").map(b => BotStore.cloneBot(b));
        });
    }
    static getProfile(userId) {
        return __awaiter(this, void 0, void 0, function* () {
            const key = "profile_" + userId;
            if (!MemoryCache_1.Cache.for(MemoryCache_1.CACHE_SIMPLE_BOTS).get(key)) {
                const client = new MatrixLiteClient_1.MatrixLiteClient(config_1.default.homeserver.accessToken);
                const profile = yield client.getProfile(userId);
                const cached = { userId, displayName: profile.displayname, avatarMxc: profile.avatar_url };
                MemoryCache_1.Cache.for(MemoryCache_1.CACHE_SIMPLE_BOTS).put(key, cached);
            }
            return MemoryCache_1.Cache.for(MemoryCache_1.CACHE_SIMPLE_BOTS).get(key);
        });
    }
    static mapAndCacheCustom(bot) {
        return __awaiter(this, void 0, void 0, function* () {
            MemoryCache_1.Cache.for(MemoryCache_1.CACHE_SIMPLE_BOTS).del("bots");
            return {
                id: bot.id,
                type: bot.type,
                name: bot.name,
                avatarUrl: bot.avatarUrl,
                description: bot.description,
                isEnabled: bot.isEnabled,
                isPublic: bot.isPublic,
                userId: bot.userId,
                accessToken: bot.accessToken,
            };
        });
    }
    static cloneBot(bot) {
        return {
            id: bot.id,
            type: bot.type,
            name: bot.name,
            avatarUrl: bot.avatarUrl,
            description: bot.description,
            isEnabled: bot.isEnabled,
            isPublic: bot.isPublic,
            userId: bot.userId,
            accessToken: bot.accessToken,
        };
    }
}
BotStore.TYPE_PREFIX = "custom_";
exports.BotStore = BotStore;
//# sourceMappingURL=BotStore.js.map