"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const sequelize_typescript_1 = require("sequelize-typescript");
const config_1 = require("../config");
const matrix_js_snippets_1 = require("matrix-js-snippets");
const User_1 = require("./models/User");
const UserScalarToken_1 = require("./models/UserScalarToken");
const Upstream_1 = require("./models/Upstream");
const WidgetRecord_1 = require("./models/WidgetRecord");
const path = require("path");
const Umzug = require("umzug");
const AppService_1 = require("./models/AppService");
const AppServiceUser_1 = require("./models/AppServiceUser");
const NebConfiguration_1 = require("./models/NebConfiguration");
const NebIntegration_1 = require("./models/NebIntegration");
const NebBotUser_1 = require("./models/NebBotUser");
const NebNotificationUser_1 = require("./models/NebNotificationUser");
const NebIntegrationConfig_1 = require("./models/NebIntegrationConfig");
const Webhook_1 = require("./models/Webhook");
const BridgeRecord_1 = require("./models/BridgeRecord");
const IrcBridgeRecord_1 = require("./models/IrcBridgeRecord");
const IrcBridgeNetwork_1 = require("./models/IrcBridgeNetwork");
const StickerPack_1 = require("./models/StickerPack");
const Sticker_1 = require("./models/Sticker");
const UserStickerPack_1 = require("./models/UserStickerPack");
const TelegramBridgeRecord_1 = require("./models/TelegramBridgeRecord");
const WebhookBridgeRecord_1 = require("./models/WebhookBridgeRecord");
const GitterBridgeRecord_1 = require("./models/GitterBridgeRecord");
const CustomSimpleBotRecord_1 = require("./models/CustomSimpleBotRecord");
const SlackBridgeRecord_1 = require("./models/SlackBridgeRecord");
const TermsRecord_1 = require("./models/TermsRecord");
const TermsTextRecord_1 = require("./models/TermsTextRecord");
const TermsSignedRecord_1 = require("./models/TermsSignedRecord");
const TermsUpstreamRecord_1 = require("./models/TermsUpstreamRecord");
class _DimensionStore {
    constructor() {
        if (process.env.DATABASE_URI || config_1.default.database.uri) {
            this.sequelize = new sequelize_typescript_1.Sequelize(process.env.DATABASE_URI || config_1.default.database.uri, {
                logging: i => matrix_js_snippets_1.LogService.verbose("DimensionStore [SQL]", i)
            });
        }
        else {
            this.sequelize = new sequelize_typescript_1.Sequelize({
                dialect: 'sqlite',
                database: "dimension",
                storage: process.env['DIMENSION_DB_PATH'] || config_1.default.database.file,
                username: "",
                password: "",
                logging: i => matrix_js_snippets_1.LogService.verbose("DimensionStore [SQL]", i)
            });
        }
        this.sequelize.addModels([
            User_1.default,
            UserScalarToken_1.default,
            Upstream_1.default,
            WidgetRecord_1.default,
            AppService_1.default,
            AppServiceUser_1.default,
            NebConfiguration_1.default,
            NebIntegration_1.default,
            NebBotUser_1.default,
            NebNotificationUser_1.default,
            NebIntegrationConfig_1.default,
            Webhook_1.default,
            BridgeRecord_1.default,
            IrcBridgeRecord_1.default,
            IrcBridgeNetwork_1.default,
            StickerPack_1.default,
            Sticker_1.default,
            UserStickerPack_1.default,
            TelegramBridgeRecord_1.default,
            WebhookBridgeRecord_1.default,
            GitterBridgeRecord_1.default,
            CustomSimpleBotRecord_1.default,
            SlackBridgeRecord_1.default,
            TermsRecord_1.default,
            TermsTextRecord_1.default,
            TermsSignedRecord_1.default,
            TermsUpstreamRecord_1.default,
        ]);
    }
    updateSchema() {
        matrix_js_snippets_1.LogService.info("DimensionStore", "Updating schema...");
        const migrator = new Umzug({
            storage: "sequelize",
            storageOptions: { sequelize: this.sequelize },
            migrations: {
                params: [this.sequelize.getQueryInterface()],
                path: path.join(__dirname, "migrations"),
            }
        });
        return migrator.up();
    }
}
exports.DimensionStore = new _DimensionStore();
//# sourceMappingURL=DimensionStore.js.map