"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const neb_1 = require("../models/neb");
const NebConfiguration_1 = require("./models/NebConfiguration");
const NebIntegration_1 = require("./models/NebIntegration");
const Upstream_1 = require("./models/Upstream");
const AppService_1 = require("./models/AppService");
const matrix_js_snippets_1 = require("matrix-js-snippets");
const NebClient_1 = require("../neb/NebClient");
const NebBotUser_1 = require("./models/NebBotUser");
const NebNotificationUser_1 = require("./models/NebNotificationUser");
const AppserviceStore_1 = require("./AppserviceStore");
const config_1 = require("../config");
const SimpleBot_1 = require("../integrations/SimpleBot");
const NebProxy_1 = require("../neb/NebProxy");
const ComplexBot_1 = require("../integrations/ComplexBot");
class NebStore {
    constructor() {
    }
    static listEnabledNebBots(simple) {
        return __awaiter(this, void 0, void 0, function* () {
            const nebConfigs = yield NebStore.getAllConfigs();
            const integrations = [];
            const hasTypes = [];
            for (const neb of nebConfigs) {
                for (const integration of neb.dbIntegrations) {
                    if (!integration.isEnabled)
                        continue;
                    const metadata = NebStore.INTEGRATIONS[integration.type];
                    if (!metadata || metadata.simple !== simple)
                        continue;
                    if (hasTypes.indexOf(integration.type) !== -1)
                        continue;
                    integrations.push({ neb, integration });
                    hasTypes.push(integration.type);
                }
            }
            return integrations;
        });
    }
    static listEnabledNebSimpleBots() {
        return __awaiter(this, void 0, void 0, function* () {
            return NebStore.listEnabledNebBots(true);
        });
    }
    static listEnabledNebComplexBots() {
        return __awaiter(this, void 0, void 0, function* () {
            return NebStore.listEnabledNebBots(false);
        });
    }
    static listSimpleBots(requestingUserId) {
        return __awaiter(this, void 0, void 0, function* () {
            const rawIntegrations = yield NebStore.listEnabledNebSimpleBots();
            return Promise.all(rawIntegrations.map((i) => __awaiter(this, void 0, void 0, function* () {
                const proxy = new NebProxy_1.NebProxy(i.neb, requestingUserId);
                let userId = null;
                try {
                    userId = yield proxy.getBotUserId(i.integration);
                }
                catch (e) {
                    matrix_js_snippets_1.LogService.error("NebStore", e);
                }
                const bot = SimpleBot_1.SimpleBot.fromNeb(i.integration, userId);
                bot.isOnline = !!userId;
                return bot;
            })));
        });
    }
    static listComplexBots(requestingUserId, roomId) {
        return __awaiter(this, void 0, void 0, function* () {
            const rawIntegrations = yield NebStore.listEnabledNebComplexBots();
            return Promise.all(rawIntegrations.map((i) => __awaiter(this, void 0, void 0, function* () {
                const proxy = new NebProxy_1.NebProxy(i.neb, requestingUserId);
                try {
                    const notifUserId = yield proxy.getNotificationUserId(i.integration, roomId);
                    const botUserId = null; // TODO: For github
                    const botConfig = yield proxy.getServiceConfiguration(i.integration, roomId);
                    const bot = new ComplexBot_1.ComplexBot(i.integration, notifUserId, botUserId, botConfig);
                    bot.isOnline = !!bot.notificationUserId;
                    return bot;
                }
                catch (e) {
                    matrix_js_snippets_1.LogService.error("NebStore", e);
                    const bot = new ComplexBot_1.ComplexBot(i.integration, null, null, null);
                    bot.isOnline = false;
                    return bot;
                }
            })));
        });
    }
    static setComplexBotConfig(requestingUserId, type, roomId, newConfig) {
        return __awaiter(this, void 0, void 0, function* () {
            const rawIntegrations = yield NebStore.listEnabledNebComplexBots();
            const integration = rawIntegrations.find(i => i.integration.type === type);
            if (!integration)
                throw new Error("Integration not found");
            const proxy = new NebProxy_1.NebProxy(integration.neb, requestingUserId);
            return proxy.setServiceConfiguration(integration.integration, roomId, newConfig);
        });
    }
    static removeSimpleBot(type, roomId, requestingUserId) {
        return __awaiter(this, void 0, void 0, function* () {
            const rawIntegrations = yield NebStore.listEnabledNebSimpleBots();
            const integration = rawIntegrations.find(i => i.integration.type === type);
            if (!integration)
                throw new Error("Integration not found");
            const proxy = new NebProxy_1.NebProxy(integration.neb, requestingUserId);
            return proxy.removeBotFromRoom(integration.integration, roomId);
        });
    }
    static getAllConfigs() {
        return __awaiter(this, void 0, void 0, function* () {
            const configs = yield NebConfiguration_1.default.findAll();
            return Promise.all((configs || []).map(c => NebStore.getConfig(c.id)));
        });
    }
    static getConfig(id) {
        return __awaiter(this, void 0, void 0, function* () {
            const nebConfig = yield NebConfiguration_1.default.findByPk(id);
            if (!nebConfig)
                throw new Error("Configuration not found");
            const integrations = yield NebIntegration_1.default.findAll({ where: { nebId: id } });
            const fullIntegrations = yield NebStore.getCompleteIntegrations(nebConfig, integrations);
            return new neb_1.NebConfig(nebConfig, fullIntegrations);
        });
    }
    static createForUpstream(upstreamId) {
        return __awaiter(this, void 0, void 0, function* () {
            const upstream = yield Upstream_1.default.findByPk(upstreamId);
            if (!upstream)
                throw new Error("Upstream not found");
            const nebConfig = yield NebConfiguration_1.default.create({
                upstreamId: upstream.id,
            });
            return NebStore.getConfig(nebConfig.id);
        });
    }
    static createForAppservice(appserviceId, adminUrl) {
        return __awaiter(this, void 0, void 0, function* () {
            const appservice = yield AppService_1.default.findByPk(appserviceId);
            if (!appservice)
                throw new Error("Appservice not found");
            const nebConfig = yield NebConfiguration_1.default.create({
                appserviceId: appservice.id,
                adminUrl: adminUrl,
            });
            return NebStore.getConfig(nebConfig.id);
        });
    }
    static getOrCreateIntegration(configurationId, integrationType) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!NebStore.INTEGRATIONS[integrationType])
                throw new Error("Integration not supported");
            const nebConfig = yield NebConfiguration_1.default.findByPk(configurationId);
            if (!nebConfig)
                throw new Error("Configuration not found");
            let integration = yield NebIntegration_1.default.findOne({ where: { nebId: nebConfig.id, type: integrationType } });
            if (!integration) {
                matrix_js_snippets_1.LogService.info("NebStore", "Creating integration " + integrationType + " for NEB " + configurationId);
                integration = yield NebIntegration_1.default.create({
                    type: integrationType,
                    name: NebStore.INTEGRATIONS[integrationType].name,
                    avatarUrl: NebStore.INTEGRATIONS[integrationType].avatarUrl,
                    description: NebStore.INTEGRATIONS[integrationType].description,
                    isEnabled: false,
                    isPublic: true,
                    nebId: configurationId,
                });
            }
            return integration;
        });
    }
    static getCompleteIntegrations(nebConfig, knownIntegrations) {
        return __awaiter(this, void 0, void 0, function* () {
            const supported = NebStore.getSupportedIntegrations(nebConfig);
            const notSupported = [];
            for (const supportedIntegration of supported) {
                let isSupported = false;
                for (const integration of knownIntegrations) {
                    if (integration.type === supportedIntegration.type) {
                        isSupported = true;
                        break;
                    }
                }
                if (!isSupported)
                    notSupported.push(supportedIntegration);
            }
            const promises = [];
            for (const missingIntegration of notSupported) {
                promises.push(NebStore.getOrCreateIntegration(nebConfig.id, missingIntegration.type));
            }
            return Promise.all(promises).then(addedIntegrations => (addedIntegrations || []).concat(knownIntegrations));
        });
    }
    static getSupportedIntegrations(nebConfig) {
        const result = [];
        for (const type of Object.keys(NebStore.INTEGRATIONS)) {
            if (nebConfig.upstreamId && NebStore.INTEGRATIONS_MODULAR_SUPPORTED.indexOf(type) === -1)
                continue;
            const integrationConfig = JSON.parse(JSON.stringify(NebStore.INTEGRATIONS[type]));
            integrationConfig["type"] = type;
            result.push(integrationConfig);
        }
        return result;
    }
    static setIntegrationEnabled(configurationId, integrationType, isEnabled) {
        return __awaiter(this, void 0, void 0, function* () {
            const integration = yield this.getOrCreateIntegration(configurationId, integrationType);
            integration.isEnabled = isEnabled;
            yield integration.save();
            const neb = yield this.getConfig(configurationId);
            if (!neb.appserviceId)
                return; // Done - nothing to do from here
            const client = new NebClient_1.NebClient(neb);
            const botUsers = yield NebBotUser_1.default.findAll({ where: { integrationId: integration.id } });
            for (const user of botUsers) {
                yield client.updateUser(user.appserviceUserId, isEnabled, true);
            }
            const notificationUsers = yield NebNotificationUser_1.default.findAll({ where: { integrationId: integration.id } });
            for (const user of notificationUsers) {
                yield client.updateUser(user.appserviceUserId, isEnabled, false);
            }
        });
    }
    static getOrCreateBotUser(configurationId, integrationType) {
        return __awaiter(this, void 0, void 0, function* () {
            const neb = yield NebStore.getConfig(configurationId);
            if (!neb.appserviceId)
                throw new Error("Instance not bound to an appservice");
            const integration = yield this.getOrCreateIntegration(configurationId, integrationType);
            const users = yield NebBotUser_1.default.findAll({ where: { integrationId: integration.id } });
            if (!users || users.length === 0) {
                const appservice = yield AppserviceStore_1.AppserviceStore.getAppservice(neb.appserviceId);
                const userId = "@" + appservice.userPrefix + "_" + integrationType + ":" + config_1.default.homeserver.name;
                const appserviceUser = yield AppserviceStore_1.AppserviceStore.getOrCreateUser(neb.appserviceId, userId);
                const client = new NebClient_1.NebClient(neb);
                yield client.updateUser(userId, integration.isEnabled, true); // creates the user in go-neb
                client.updateUserProfile(userId, integration.name, config_1.default.goneb.avatars[integration.type]).then(() => {
                    matrix_js_snippets_1.LogService.info("NebStore", "Updated profile for " + userId);
                }).catch(err => {
                    matrix_js_snippets_1.LogService.error("NebStore", "Error updating profile for " + userId);
                    matrix_js_snippets_1.LogService.error("NebStore", err);
                });
                const serviceId = appservice.id + "_integration_" + integrationType;
                return NebBotUser_1.default.create({
                    serviceId: serviceId,
                    appserviceUserId: appserviceUser.id,
                    integrationId: integration.id,
                });
            }
            return users[0];
        });
    }
    static getOrCreateNotificationUser(configurationId, integrationType, forUserId) {
        return __awaiter(this, void 0, void 0, function* () {
            const neb = yield NebStore.getConfig(configurationId);
            if (!neb.appserviceId)
                throw new Error("Instance not bound to an appservice");
            const integration = yield this.getOrCreateIntegration(configurationId, integrationType);
            const users = yield NebNotificationUser_1.default.findAll({ where: { integrationId: integration.id, ownerId: forUserId } });
            if (!users || users.length === 0) {
                const safeUserId = AppserviceStore_1.AppserviceStore.getSafeUserId(forUserId);
                const appservice = yield AppserviceStore_1.AppserviceStore.getAppservice(neb.appserviceId);
                const userId = "@" + appservice.userPrefix + "_" + integrationType + "_notifications_" + safeUserId + ":" + config_1.default.homeserver.name;
                const appserviceUser = yield AppserviceStore_1.AppserviceStore.getOrCreateUser(neb.appserviceId, userId);
                const client = new NebClient_1.NebClient(neb);
                yield client.updateUser(userId, integration.isEnabled, false); // creates the user in go-neb
                client.updateUserProfile(userId, integration.name + " Notifications [" + forUserId + "]", config_1.default.goneb.avatars[integration.type]).then(() => {
                    matrix_js_snippets_1.LogService.info("NebStore", "Updated profile for " + userId);
                }).catch(err => {
                    matrix_js_snippets_1.LogService.error("NebStore", "Error updating profile for " + userId);
                    matrix_js_snippets_1.LogService.error("NebStore", err);
                });
                const serviceId = appservice.id + "_integration_" + integrationType + "_notifications_" + safeUserId;
                return NebNotificationUser_1.default.create({
                    serviceId: serviceId,
                    appserviceUserId: appserviceUser.id,
                    integrationId: integration.id,
                    ownerId: forUserId,
                });
            }
            return users[0];
        });
    }
    static setIntegrationConfig(configurationId, integrationType, newConfig) {
        return __awaiter(this, void 0, void 0, function* () {
            const botUser = yield NebStore.getOrCreateBotUser(configurationId, integrationType);
            const neb = yield NebStore.getConfig(configurationId);
            const client = new NebClient_1.NebClient(neb);
            return client.setServiceConfig(botUser.serviceId, botUser.appserviceUserId, integrationType, newConfig);
        });
    }
    static getIntegrationConfig(configurationId, integrationType) {
        return __awaiter(this, void 0, void 0, function* () {
            const botUser = yield NebStore.getOrCreateBotUser(configurationId, integrationType);
            const neb = yield NebStore.getConfig(configurationId);
            const client = new NebClient_1.NebClient(neb);
            return client.getServiceConfig(botUser.serviceId);
        });
    }
}
NebStore.INTEGRATIONS_MODULAR_SUPPORTED = ["giphy", "guggy", "github", "google", "imgur", "rss", "travisci", "wikipedia"];
NebStore.INTEGRATIONS = {
    // TODO: Support Circle CI
    // "circleci": {
    //     name: "Circle CI",
    //     avatarUrl: "/img/avatars/circleci.png",
    //     description: "Announces build results from Circle CI to the room.",
    //     simple: false,
    // },
    "echo": {
        name: "Echo",
        avatarUrl: "/img/avatars/echo.png",
        description: "Repeats text given to it from !echo",
        simple: true,
    },
    "giphy": {
        name: "Giphy",
        avatarUrl: "/img/avatars/giphy.png",
        description: "Posts a GIF from Giphy using !giphy <query>",
        simple: true,
    },
    "guggy": {
        name: "Guggy",
        avatarUrl: "/img/avatars/guggy.png",
        description: "Send a reaction GIF using !guggy <query>",
        simple: true,
    },
    // TODO: Support Github
    // "github": {
    //     name: "Github",
    //     avatarUrl: "/img/avatars/github.png",
    //     description: "Github issue management and announcements for a repository",
    //     simple: false,
    // },
    "google": {
        name: "Google",
        avatarUrl: "/img/avatars/google.png",
        description: "Searches Google Images using !google image <query>",
        simple: true,
    },
    "imgur": {
        name: "Imgur",
        avatarUrl: "/img/avatars/imgur.png",
        description: "Searches and posts images from Imgur using !imgur <query>",
        simple: true,
    },
    // TODO: Support JIRA
    // "jira": {
    //     name: "Jira",
    //     avatarUrl: "/img/avatars/jira.png",
    //     description: "Jira issue management and announcements for a project",
    //     simple: false,
    // },
    "rss": {
        name: "RSS",
        avatarUrl: "/img/avatars/rssbot.png",
        description: "Announces changes to RSS feeds in the room",
        simple: false,
    },
    "travisci": {
        name: "Travis CI",
        avatarUrl: "/img/avatars/travisci.png",
        description: "Announces build results from Travis CI to the room",
        simple: false,
    },
    "wikipedia": {
        name: "Wikipedia",
        avatarUrl: "/img/avatars/wikipedia.png",
        description: "Searches wikipedia using !wikipedia <query>",
        simple: true,
    },
};
exports.NebStore = NebStore;
//# sourceMappingURL=NebStore.js.map