"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const matrix_js_snippets_1 = require("matrix-js-snippets");
const config_1 = require("./config");
const DimensionStore_1 = require("./db/DimensionStore");
const Webserver_1 = require("./api/Webserver");
const version_1 = require("./version");
const MatrixStickerBot_1 = require("./matrix/MatrixStickerBot");
const BotSdk = require("matrix-bot-sdk");
const User_1 = require("./db/models/User");
matrix_js_snippets_1.LogService.configure(config_1.default.logging);
matrix_js_snippets_1.LogService.info("index", "Starting dimension " + version_1.CURRENT_VERSION);
// Redirect the bot-sdk logger to our logger
BotSdk.LogService.setLogger({
    debug: (module, ...args) => args.map(a => matrix_js_snippets_1.LogService.info("BotSdk-" + module, a)),
    info: (module, ...args) => args.map(a => matrix_js_snippets_1.LogService.info("BotSdk-" + module, a)),
    warn: (module, ...args) => args.map(a => matrix_js_snippets_1.LogService.warn("BotSdk-" + module, a)),
    error: (module, ...args) => args.map(a => matrix_js_snippets_1.LogService.error("BotSdk-" + module, a)),
});
function startup() {
    return __awaiter(this, void 0, void 0, function* () {
        yield DimensionStore_1.DimensionStore.updateSchema();
        const webserver = new Webserver_1.default();
        yield webserver.start();
        const userId = yield MatrixStickerBot_1.MatrixStickerBot.getUserId();
        const users = yield User_1.default.findAll({ where: { userId: userId, isSelfBot: false } });
        if (users.length > 0) {
            matrix_js_snippets_1.LogService.error("index", "The access token configured for Dimension belongs to a user which is also " +
                "a user known to Dimension. This usually indicates that the access token is not a dedicated user " +
                "account for Dimension. To prevent potential confusion to this user, Dimension will refuse to start " +
                "until the access token given belongs to a dedicated user.");
            throw new Error("Access token belongs to a real user. See logs for details.");
        }
        matrix_js_snippets_1.LogService.info("index", "Sticker bot is using utility account, registered as " + userId);
        yield MatrixStickerBot_1.MatrixStickerBot.start();
    });
}
startup()
    .then(() => matrix_js_snippets_1.LogService.info("index", "Dimension is ready!"))
    .catch((e) => {
    console.error(e);
    process.exit(1);
});
//# sourceMappingURL=index.js.map