"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const helpers_1 = require("./helpers");
const config_1 = require("../config");
const request = require("request");
const matrix_js_snippets_1 = require("matrix-js-snippets");
class MatrixLiteClient {
    constructor(accessToken) {
        this.accessToken = accessToken;
    }
    getUrlPreview(url) {
        return __awaiter(this, void 0, void 0, function* () {
            return helpers_1.doClientApiCall("GET", "/_matrix/media/r0/preview_url", { access_token: this.accessToken, url: url });
        });
    }
    getThumbnailUrl(serverName, contentId, width, height, method, isAnimated) {
        return __awaiter(this, void 0, void 0, function* () {
            let baseUrl = config_1.default.homeserver.mediaUrl;
            if (!baseUrl)
                baseUrl = config_1.default.homeserver.clientServerUrl;
            if (baseUrl.endsWith("/"))
                baseUrl = baseUrl.substring(0, baseUrl.length - 1);
            // DO NOT RETURN THE ACCESS TOKEN.
            return baseUrl + `/_matrix/media/r0/thumbnail/${serverName}/${contentId}?width=${width}&height=${height}&method=${method}&animated=${isAnimated}`;
        });
    }
    whoAmI() {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield helpers_1.doClientApiCall("GET", "/_matrix/client/r0/account/whoami", { access_token: this.accessToken });
            return response['user_id'];
        });
    }
    getOpenId() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield helpers_1.doClientApiCall("POST", `/_matrix/client/r0/user/${yield this.whoAmI()}/openid/request_token`, { access_token: this.accessToken }, {});
        });
    }
    leaveRoom(roomId) {
        return __awaiter(this, void 0, void 0, function* () {
            return helpers_1.doClientApiCall("POST", "/_matrix/client/r0/rooms/" + roomId + "/leave", { access_token: this.accessToken });
        });
    }
    getProfile(userId) {
        return __awaiter(this, void 0, void 0, function* () {
            return helpers_1.doClientApiCall("GET", "/_matrix/client/r0/profile/" + userId, { access_token: this.accessToken });
        });
    }
    getDisplayName() {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield helpers_1.doClientApiCall("GET", "/_matrix/client/r0/profile/" + (yield this.whoAmI()) + "/displayname", { access_token: this.accessToken });
            return response['displayname'];
        });
    }
    getAvatarUrl() {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield helpers_1.doClientApiCall("GET", "/_matrix/client/r0/profile/" + (yield this.whoAmI()) + "/avatar_url", { access_token: this.accessToken });
            return response['avatar_url'];
        });
    }
    setDisplayName(newDisplayName) {
        return __awaiter(this, void 0, void 0, function* () {
            return helpers_1.doClientApiCall("PUT", "/_matrix/client/r0/profile/" + (yield this.whoAmI()) + "/displayname", { access_token: this.accessToken }, { displayname: newDisplayName });
        });
    }
    setAvatarUrl(newUrl) {
        return __awaiter(this, void 0, void 0, function* () {
            return helpers_1.doClientApiCall("PUT", "/_matrix/client/r0/profile/" + (yield this.whoAmI()) + "/avatar_url", { access_token: this.accessToken }, { avatar_url: newUrl });
        });
    }
    upload(content, contentType) {
        return __awaiter(this, void 0, void 0, function* () {
            return helpers_1.doClientApiCall("POST", "/_matrix/media/r0/upload", { access_token: this.accessToken }, content, contentType).then(r => r["content_uri"]);
        });
    }
    uploadFromUrl(url, contentType) {
        return __awaiter(this, void 0, void 0, function* () {
            const buffer = yield this.downloadFromUrl(url);
            return this.upload(buffer, contentType);
        });
    }
    downloadFromUrl(url) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                request({
                    method: "GET",
                    url: url,
                    encoding: null,
                }, (err, res, _body) => {
                    if (err) {
                        matrix_js_snippets_1.LogService.error("MatrixLiteClient", "Error downloading file from " + url);
                        matrix_js_snippets_1.LogService.error("MatrixLiteClient", err);
                        reject(err);
                    }
                    else if (res.statusCode !== 200) {
                        matrix_js_snippets_1.LogService.error("MatrixLiteClient", "Got status code " + res.statusCode + " while calling url " + url);
                        reject(new Error("Error in request: invalid status code"));
                    }
                    else {
                        resolve(res.body);
                    }
                });
            });
        });
    }
}
exports.MatrixLiteClient = MatrixLiteClient;
//# sourceMappingURL=MatrixLiteClient.js.map