"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const matrix_bot_sdk_1 = require("matrix-bot-sdk");
const config_1 = require("../config");
const matrix_js_snippets_1 = require("matrix-js-snippets");
const StickerPack_1 = require("../db/models/StickerPack");
const Sticker_1 = require("../db/models/Sticker");
const MatrixLiteClient_1 = require("./MatrixLiteClient");
const MemoryCache_1 = require("../MemoryCache");
const LicenseMap_1 = require("../utils/LicenseMap");
class _MatrixStickerBot {
    constructor() {
        this.updatingPackIds = {};
        this.client = new matrix_bot_sdk_1.MatrixClient(config_1.default.homeserver.clientServerUrl, config_1.default.homeserver.accessToken, new matrix_bot_sdk_1.SimpleFsStorageProvider(config_1.default.database.botData));
        this.client.setJoinStrategy(new matrix_bot_sdk_1.SimpleRetryJoinStrategy());
        this.client.on("room.event", this.onEvent.bind(this));
        matrix_bot_sdk_1.AutojoinUpgradedRoomsMixin.setupOnClient(this.client);
    }
    start() {
        return this.client.start().then(() => matrix_js_snippets_1.LogService.info("MatrixStickerBot", "Sticker bot started"));
    }
    getUserId() {
        return this.client.getUserId();
    }
    getOpenId() {
        const liteClient = new MatrixLiteClient_1.MatrixLiteClient(config_1.default.homeserver.accessToken);
        return liteClient.getOpenId();
    }
    onEvent(roomId, event) {
        return __awaiter(this, void 0, void 0, function* () {
            matrix_js_snippets_1.LogService.info("MatrixStickerBot", `Event ${event.type} in ${roomId}`);
            if (event.type !== "io.t2bot.stickers.metadata" || event.state_key !== "")
                return;
            const canonicalAlias = yield this.client.getRoomStateEvent(roomId, "m.room.canonical_alias", "");
            const stickerPacks = yield StickerPack_1.default.findAll({ where: { trackingRoomAlias: canonicalAlias.alias } });
            if (stickerPacks.length > 0) {
                return this.updateStickersInPacks(stickerPacks, roomId);
            }
        });
    }
    trackStickerpack(alias) {
        return this.client.joinRoom(alias).then((roomId) => __awaiter(this, void 0, void 0, function* () {
            const stickerPacks = yield StickerPack_1.default.findAll({ where: { trackingRoomAlias: alias } });
            if (stickerPacks.length > 0) {
                return this.updateStickersInPacks(stickerPacks, roomId);
            }
            else {
                const pack = yield StickerPack_1.default.create({
                    type: "stickerpack",
                    name: "PLACEHOLDER",
                    description: "PLACEHOLDER",
                    avatarUrl: "mxc://localhost/NotYetLoaded",
                    isEnabled: false,
                    isPublic: true,
                    authorType: "matrix",
                    authorName: yield this.getUserId(),
                    authorReference: "https://matrix.to/#/" + (yield this.getUserId()),
                    license: "Imported",
                    licensePath: "/licenses/general-imported.txt",
                    trackingRoomAlias: alias,
                });
                return this.updateStickersInPacks([pack], roomId);
            }
        }));
    }
    updateStickersInPacks(stickerPacks, roomId) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.updatingPackIds[roomId])
                return;
            this.updatingPackIds[roomId] = true;
            try {
                const nameEvent = yield this.client.getRoomStateEvent(roomId, "m.room.name", "");
                if (!nameEvent)
                    return null;
                const canconicalAliasEvent = yield this.client.getRoomStateEvent(roomId, "m.room.canonical_alias", "");
                if (!canconicalAliasEvent)
                    return null;
                const packEvent = yield this.client.getRoomStateEvent(roomId, "io.t2bot.stickers.metadata", "");
                if (!packEvent)
                    return null;
                let authorDisplayName = packEvent.creatorId;
                try {
                    const profile = yield this.client.getUserProfile(packEvent.creatorId);
                    if (profile && profile.displayname)
                        authorDisplayName = profile.displayname;
                }
                catch (e) {
                    matrix_js_snippets_1.LogService.warn("MatrixStickerBot", e);
                }
                const mx = new MatrixLiteClient_1.MatrixLiteClient(config_1.default.homeserver.accessToken);
                const stickerEvents = [];
                for (const stickerId of packEvent.activeStickers) {
                    const stickerEvent = yield this.client.getRoomStateEvent(roomId, "io.t2bot.stickers.sticker", stickerId);
                    if (!stickerEvent)
                        continue;
                    const mxc = stickerEvent.contentUri;
                    if (!mxc.startsWith("mxc://"))
                        continue;
                    const serverName = mxc.substring("mxc://".length).split("/")[0];
                    const contentId = mxc.substring("mxc://".length).split("/")[1];
                    stickerEvent.thumbMxc = yield mx.uploadFromUrl(yield mx.getThumbnailUrl(serverName, contentId, 512, 512, "scale", false), "image/png");
                    stickerEvents.push(stickerEvent);
                }
                const creatorId = packEvent.creatorId;
                const authorName = packEvent.authorName || packEvent.creatorId;
                const authorUrl = packEvent.authorUrl || `https://matrix.to/#/${packEvent.creatorId}`;
                const authorIsCreator = creatorId === authorName;
                let license = LicenseMap_1.LicenseMap.find(packEvent.license || "");
                if (!license)
                    license = LicenseMap_1.LicenseMap.find(LicenseMap_1.LicenseMap.LICENSE_IMPORTED);
                for (const pack of stickerPacks) {
                    pack.isEnabled = true;
                    pack.trackingRoomAlias = canconicalAliasEvent.alias;
                    pack.name = nameEvent.name;
                    if (authorIsCreator) {
                        pack.authorType = "matrix";
                        pack.authorReference = "https://matrix.to/#/" + packEvent.creatorId;
                        pack.authorName = authorDisplayName;
                    }
                    else {
                        pack.authorType = "website";
                        pack.authorReference = authorUrl;
                        pack.authorName = authorName;
                    }
                    pack.description = "Matrix sticker pack created by " + authorDisplayName;
                    pack.license = license.name;
                    pack.licensePath = license.url;
                    if (stickerEvents.length > 0)
                        pack.avatarUrl = stickerEvents[0].contentUri;
                    yield pack.save();
                    const existingStickers = yield Sticker_1.default.findAll({ where: { packId: pack.id } });
                    for (const sticker of existingStickers)
                        yield sticker.destroy();
                    for (const stickerEvent of stickerEvents) {
                        yield Sticker_1.default.create({
                            packId: pack.id,
                            name: stickerEvent.description,
                            description: stickerEvent.description,
                            imageMxc: stickerEvent.contentUri,
                            thumbnailMxc: stickerEvent.thumbMxc,
                            thumbnailWidth: 512,
                            thumbnailHeight: 512,
                            mimetype: "image/png",
                        });
                    }
                }
                matrix_js_snippets_1.LogService.info("MatrixStickerBot", `Updated ${stickerPacks.length} stickerpacks`);
                MemoryCache_1.Cache.for(MemoryCache_1.CACHE_STICKERS).clear();
            }
            catch (e) {
                matrix_js_snippets_1.LogService.error("MatrixStickerBot", e);
            }
            delete this.updatingPackIds[roomId];
        });
    }
}
exports.MatrixStickerBot = new _MatrixStickerBot();
//# sourceMappingURL=MatrixStickerBot.js.map