"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const AppserviceStore_1 = require("../db/AppserviceStore");
const config_1 = require("../config");
const matrix_js_snippets_1 = require("matrix-js-snippets");
const request = require("request");
const MatrixLiteClient_1 = require("../matrix/MatrixLiteClient");
class NebClient {
    constructor(neb) {
        this.neb = neb;
    }
    getAccessToken(userId) {
        return __awaiter(this, void 0, void 0, function* () {
            const user = yield AppserviceStore_1.AppserviceStore.getOrCreateUser(this.neb.appserviceId, userId);
            return user.accessToken;
        });
    }
    static getNebType(type) {
        if (type === "rss")
            return "rssbot";
        if (type === "travisci")
            return "travis-ci";
        return type;
    }
    updateUser(userId, isEnabled, sync = true, autoAcceptInvites = true) {
        return __awaiter(this, void 0, void 0, function* () {
            const nebRequest = {
                UserID: userId,
                HomeserverURL: config_1.default.homeserver.clientServerUrl,
                AccessToken: (isEnabled ? yield this.getAccessToken(userId) : "DISABLED"),
                Sync: isEnabled ? sync : false,
                AutoJoinRooms: autoAcceptInvites
            };
            return this.doRequest("/admin/configureClient", nebRequest);
        });
    }
    setServiceConfig(serviceId, userId, type, serviceConfig) {
        return __awaiter(this, void 0, void 0, function* () {
            const nebRequest = {
                ID: serviceId,
                Type: NebClient.getNebType(type),
                UserID: userId,
                Config: serviceConfig,
            };
            return this.doRequest("/admin/configureService", nebRequest);
        });
    }
    getServiceConfig(serviceId) {
        return __awaiter(this, void 0, void 0, function* () {
            const nebRequest = { ID: serviceId };
            try {
                const service = yield this.doRequest("/admin/getService", nebRequest);
                return service.Config;
            }
            catch (err) {
                matrix_js_snippets_1.LogService.error("NebClient", err);
                return {};
            }
        });
    }
    updateUserProfile(userId, displayName, avatarUrl) {
        return __awaiter(this, void 0, void 0, function* () {
            const accessToken = yield this.getAccessToken(userId);
            const client = new MatrixLiteClient_1.MatrixLiteClient(accessToken);
            const currentDisplayName = yield client.getDisplayName();
            const currentAvatarUrl = yield client.getAvatarUrl();
            if (currentDisplayName !== displayName)
                yield client.setDisplayName(displayName);
            if (currentAvatarUrl !== avatarUrl)
                yield client.setAvatarUrl(avatarUrl);
        });
    }
    doRequest(endpoint, body) {
        const adminUrl = (this.neb.adminUrl.endsWith("/") ? this.neb.adminUrl.substring(0, this.neb.adminUrl.length - 1) : this.neb.adminUrl);
        if (!endpoint.startsWith("/"))
            endpoint = "/" + endpoint;
        matrix_js_snippets_1.LogService.info("NebClient", "Doing NEB call: " + adminUrl + endpoint);
        return new Promise((resolve, reject) => {
            request({
                method: "POST",
                url: adminUrl + endpoint,
                json: body,
            }, (err, res, _body) => {
                if (err) {
                    matrix_js_snippets_1.LogService.error("NebClient", "Error performing request");
                    matrix_js_snippets_1.LogService.error("NebClient", err);
                    reject(err);
                }
                else if (res.statusCode !== 200) {
                    matrix_js_snippets_1.LogService.error("NebClient", "Got status code " + res.statusCode + " while performing request");
                    matrix_js_snippets_1.LogService.error("NebClient", res.body);
                    reject(new Error("Request error"));
                }
                else {
                    resolve(res.body);
                }
            });
        });
    }
}
exports.NebClient = NebClient;
//# sourceMappingURL=NebClient.js.map