"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const NebStore_1 = require("../db/NebStore");
const matrix_js_snippets_1 = require("matrix-js-snippets");
const request = require("request");
const Upstream_1 = require("../db/models/Upstream");
const UserScalarToken_1 = require("../db/models/UserScalarToken");
const NebClient_1 = require("./NebClient");
const AppserviceStore_1 = require("../db/AppserviceStore");
const MatrixAppserviceClient_1 = require("../matrix/MatrixAppserviceClient");
const NebIntegrationConfig_1 = require("../db/models/NebIntegrationConfig");
const Webhook_1 = require("../db/models/Webhook");
const randomString = require("random-string");
class NebProxy {
    constructor(neb, requestingUserId) {
        this.neb = neb;
        this.requestingUserId = requestingUserId;
    }
    getBotUserId(integration) {
        return __awaiter(this, void 0, void 0, function* () {
            if (integration.nebId !== this.neb.id)
                throw new Error("Integration is not for this NEB proxy");
            if (this.neb.upstreamId) {
                try {
                    const response = yield this.doUpstreamRequest("/integrations/" + NebClient_1.NebClient.getNebType(integration.type));
                    return response.bot_user_id;
                }
                catch (err) {
                    matrix_js_snippets_1.LogService.error("NebProxy", err);
                    return null;
                }
            }
            else {
                return (yield NebStore_1.NebStore.getOrCreateBotUser(this.neb.id, integration.type)).appserviceUserId;
            }
        });
    }
    getNotificationUserId(integration, inRoomId) {
        return __awaiter(this, void 0, void 0, function* () {
            if (integration.nebId !== this.neb.id)
                throw new Error("Integration is not for this NEB proxy");
            if (this.neb.upstreamId) {
                try {
                    const response = yield this.doUpstreamRequest("/integrations/" + NebClient_1.NebClient.getNebType(integration.type), {
                        room_id: inRoomId,
                    });
                    return response.bot_user_id;
                }
                catch (err) {
                    matrix_js_snippets_1.LogService.error("NebProxy", err);
                    return null;
                }
            }
            else {
                return (yield NebStore_1.NebStore.getOrCreateNotificationUser(this.neb.id, integration.type, this.requestingUserId)).appserviceUserId;
            }
        });
    }
    getServiceConfiguration(integration, inRoomId) {
        return __awaiter(this, void 0, void 0, function* () {
            if (integration.nebId !== this.neb.id)
                throw new Error("Integration is not for this NEB proxy");
            let result = null;
            if (this.neb.upstreamId) {
                try {
                    const response = yield this.doUpstreamRequest("/integrations/" + NebClient_1.NebClient.getNebType(integration.type), {
                        room_id: inRoomId,
                    });
                    if (integration.type === "rss")
                        result = yield this.parseUpstreamRssConfiguration(response.integrations);
                    else if (integration.type === "travisci")
                        result = yield this.parseUpstreamTravisCiConfiguration(response.integrations);
                }
                catch (err) {
                    matrix_js_snippets_1.LogService.error("NebProxy", err);
                }
            }
            else {
                const serviceConfig = yield NebIntegrationConfig_1.default.findOne({
                    where: {
                        integrationId: integration.id,
                        roomId: inRoomId,
                    },
                });
                result = serviceConfig ? JSON.parse(serviceConfig.jsonContent) : {};
            }
            if (!result)
                result = {};
            if (integration.type === "travisci") {
                let repos = result.rooms ? result.rooms[inRoomId] : {};
                if (!repos)
                    repos = {};
                // Replace the entire result to better represent how this should be sent
                result = {
                    webhookId: yield this.getWebhookId("travisci"),
                    repos: repos,
                };
            }
            return result;
        });
    }
    setServiceConfiguration(integration, inRoomId, newConfig) {
        return __awaiter(this, void 0, void 0, function* () {
            if (integration.nebId !== this.neb.id)
                throw new Error("Integration is not for this NEB proxy");
            if (!this.neb.upstreamId) {
                const serviceConfig = yield NebIntegrationConfig_1.default.findOne({
                    where: {
                        integrationId: integration.id,
                        roomId: inRoomId,
                    },
                });
                if (serviceConfig) {
                    serviceConfig.jsonContent = JSON.stringify(newConfig);
                    yield serviceConfig.save();
                }
                else {
                    yield NebIntegrationConfig_1.default.create({
                        integrationId: integration.id,
                        roomId: inRoomId,
                        jsonContent: JSON.stringify(newConfig),
                    });
                }
            }
            if (integration.type === "rss")
                yield this.updateRssConfiguration(inRoomId, newConfig);
            else if (integration.type === "travisci")
                yield this.updateTravisCiConfiguration(inRoomId, newConfig);
            else
                throw new Error("Cannot update go-neb: unrecognized type");
        });
    }
    parseUpstreamRssConfiguration(integrations) {
        if (!integrations)
            return { feeds: {} };
        const result = { feeds: {} };
        for (const integration of integrations) {
            const userId = integration.user_id;
            const feeds = integration.config ? integration.config.feeds : {};
            if (!userId || !feeds)
                continue;
            const urls = Object.keys(feeds);
            urls.forEach(u => result.feeds[u] = { addedByUserId: userId });
        }
        return result;
    }
    parseUpstreamTravisCiConfiguration(integrations) {
        if (!integrations)
            return { rooms: {}, webhookUrl: null };
        const result = { rooms: {}, webhookUrl: null };
        for (const integration of integrations) {
            if (!integration.user_id || !integration.config || !integration.config.rooms)
                continue;
            const userId = integration.user_id;
            if (userId === this.requestingUserId && integration.config.webhook_url && !result.webhookUrl)
                result.webhookUrl = integration.config.webhook_url;
            const roomIds = Object.keys(integration.config.rooms);
            for (const roomId of roomIds) {
                if (!result.rooms[roomId])
                    result.rooms[roomId] = {};
                const repoKeys = Object.keys(integration.config.rooms[roomId].repos || {});
                for (const repoKey of repoKeys) {
                    result.rooms[roomId][repoKey] = {
                        template: integration.config.rooms[roomId].repos[repoKey].template,
                        addedByUserId: userId,
                    };
                }
            }
        }
        return result;
    }
    updateRssConfiguration(roomId, newOpts) {
        return __awaiter(this, void 0, void 0, function* () {
            const feedUrls = Object.keys(newOpts.feeds).filter(f => newOpts.feeds[f].addedByUserId === this.requestingUserId);
            const newConfig = { feeds: {} };
            let currentConfig = { feeds: {} };
            if (this.neb.upstreamId) {
                const response = yield this.doUpstreamRequest("/integrations/rssbot", { room_id: roomId });
                currentConfig = yield this.parseUpstreamRssConfiguration(response.integrations);
            }
            else {
                const client = new NebClient_1.NebClient(this.neb);
                const notifUser = yield NebStore_1.NebStore.getOrCreateNotificationUser(this.neb.id, "rss", this.requestingUserId);
                currentConfig = yield client.getServiceConfig(notifUser.serviceId);
                if (feedUrls.length === 0) {
                    const appserviceClient = new MatrixAppserviceClient_1.MatrixAppserviceClient(yield AppserviceStore_1.AppserviceStore.getAppservice(this.neb.appserviceId));
                    yield appserviceClient.leaveRoom(notifUser.appserviceUserId, roomId);
                }
            }
            if (!currentConfig || !currentConfig.feeds)
                currentConfig = { feeds: {} };
            const allUrls = feedUrls.concat(Object.keys(currentConfig.feeds));
            for (const feedUrl of allUrls) {
                let feed = currentConfig.feeds[feedUrl];
                if (!feed)
                    feed = { poll_interval_mins: 60, rooms: [] };
                if (!feed.rooms)
                    feed.rooms = [];
                const hasRoom = feed.rooms.indexOf(roomId) !== -1;
                const isEnabled = feedUrls.indexOf(feedUrl) !== -1;
                if (hasRoom && !isEnabled) {
                    feed.rooms.splice(feed.rooms.indexOf(roomId), 1);
                }
                else if (!hasRoom && isEnabled) {
                    feed.rooms.push(roomId);
                }
                if (feed.rooms.length > 0) {
                    newConfig.feeds[feedUrl] = {
                        poll_interval_mins: feed.poll_interval_mins,
                        rooms: feed.rooms,
                    };
                }
            }
            if (this.neb.upstreamId) {
                yield this.doUpstreamRequest("/integrations/rssbot/configureService", {
                    room_id: roomId,
                    feeds: newConfig.feeds,
                });
            }
            else {
                const client = new NebClient_1.NebClient(this.neb);
                const notifUser = yield NebStore_1.NebStore.getOrCreateNotificationUser(this.neb.id, "rss", this.requestingUserId);
                yield client.setServiceConfig(notifUser.serviceId, notifUser.appserviceUserId, "rssbot", newConfig);
            }
        });
    }
    updateTravisCiConfiguration(roomId, newOpts) {
        return __awaiter(this, void 0, void 0, function* () {
            const repoKeys = Object.keys(newOpts.repos).filter(f => newOpts.repos[f].addedByUserId === this.requestingUserId);
            let newConfig = { rooms: {} };
            if (!this.neb.upstreamId) {
                const notifUser = yield NebStore_1.NebStore.getOrCreateNotificationUser(this.neb.id, "travisci", this.requestingUserId);
                const client = new NebClient_1.NebClient(this.neb);
                newConfig = yield client.getServiceConfig(notifUser.serviceId); // So we don't accidentally clear other rooms
                if (repoKeys.length === 0) {
                    const appserviceClient = new MatrixAppserviceClient_1.MatrixAppserviceClient(yield AppserviceStore_1.AppserviceStore.getAppservice(this.neb.appserviceId));
                    yield appserviceClient.leaveRoom(notifUser.appserviceUserId, roomId);
                }
            }
            // Reset the current room's configuration so we don't keep artifacts.
            newConfig.rooms[roomId] = { repos: {} };
            const roomReposConf = newConfig.rooms[roomId].repos;
            for (const repoKey of repoKeys) {
                roomReposConf[repoKey] = {
                    template: newOpts.repos[repoKey].template,
                };
            }
            if (this.neb.upstreamId) {
                yield this.doUpstreamRequest("/integrations/travis-ci/configureService", {
                    room_id: roomId,
                    rooms: newConfig.rooms,
                });
                // Annoyingly, we don't get any kind of feedback for the webhook - we have to re-request it
                const response = yield this.doUpstreamRequest("/integrations/travis-ci", {
                    room_id: roomId,
                });
                const parsed = this.parseUpstreamTravisCiConfiguration(response.integrations);
                if (parsed && parsed.webhookUrl)
                    yield this.setWebhookTarget("travisci", parsed.webhookUrl);
            }
            else {
                const client = new NebClient_1.NebClient(this.neb);
                const notifUser = yield NebStore_1.NebStore.getOrCreateNotificationUser(this.neb.id, "travisci", this.requestingUserId);
                const result = yield client.setServiceConfig(notifUser.serviceId, notifUser.appserviceUserId, "travis-ci", newConfig);
                if (result['NewConfig'] && result['NewConfig']['webhook_url'])
                    yield this.setWebhookTarget("travisci", result['NewConfig']['webhook_url']);
            }
        });
    }
    removeBotFromRoom(integration, roomId) {
        return __awaiter(this, void 0, void 0, function* () {
            if (integration.nebId !== this.neb.id)
                throw new Error("Integration is not for this NEB proxy");
            if (this.neb.upstreamId) {
                yield this.doUpstreamRequest("/removeIntegration", { type: integration.type, room_id: roomId });
            }
            else {
                const appservice = yield AppserviceStore_1.AppserviceStore.getAppservice(this.neb.appserviceId);
                const client = new MatrixAppserviceClient_1.MatrixAppserviceClient(appservice);
                yield client.leaveRoom(yield this.getBotUserId(integration), roomId);
            }
        });
    }
    getWebhookId(serviceId) {
        return __awaiter(this, void 0, void 0, function* () {
            // We add a bit of uniqueness to the service ID to avoid conflicts
            serviceId = serviceId + "_" + this.neb.id;
            let webhook = yield Webhook_1.default.findOne({
                where: {
                    purposeId: serviceId,
                    ownerUserId: this.requestingUserId
                }
            }).catch(() => null);
            if (!webhook) {
                webhook = yield Webhook_1.default.create({
                    hookId: randomString({ length: 160 }),
                    ownerUserId: this.requestingUserId,
                    purposeId: serviceId,
                    targetUrl: null,
                });
            }
            return webhook.hookId;
        });
    }
    setWebhookTarget(serviceId, targetUrl) {
        return __awaiter(this, void 0, void 0, function* () {
            const webhookId = yield this.getWebhookId(serviceId);
            const webhook = yield Webhook_1.default.findByPk(webhookId);
            webhook.targetUrl = targetUrl;
            return webhook.save();
        });
    }
    doUpstreamRequest(endpoint, body) {
        return __awaiter(this, void 0, void 0, function* () {
            const upstream = yield Upstream_1.default.findByPk(this.neb.upstreamId);
            const token = yield UserScalarToken_1.default.findOne({
                where: {
                    upstreamId: upstream.id,
                    isDimensionToken: false,
                    userId: this.requestingUserId,
                },
            });
            const apiUrl = upstream.apiUrl.endsWith("/") ? upstream.apiUrl.substring(0, upstream.apiUrl.length - 1) : upstream.apiUrl;
            const url = apiUrl + (endpoint.startsWith("/") ? endpoint : "/" + endpoint);
            matrix_js_snippets_1.LogService.info("NebProxy", "Doing upstream NEB request: " + url);
            return new Promise((resolve, reject) => {
                request({
                    method: "POST",
                    url: url,
                    qs: { scalar_token: token.scalarToken },
                    json: body,
                }, (err, res, _body) => {
                    if (err) {
                        matrix_js_snippets_1.LogService.error("NebProxy", "Error calling" + url);
                        matrix_js_snippets_1.LogService.error("NebProxy", err);
                        reject(err);
                    }
                    else if (res.statusCode !== 200) {
                        matrix_js_snippets_1.LogService.error("NebProxy", "Got status code " + res.statusCode + " when calling " + url);
                        matrix_js_snippets_1.LogService.error("NebProxy", res.body);
                        reject(new Error("Request failed"));
                    }
                    else {
                        if (typeof (res.body) === "string")
                            res.body = JSON.parse(res.body);
                        resolve(res.body);
                    }
                });
            });
        });
    }
}
exports.NebProxy = NebProxy;
//# sourceMappingURL=NebProxy.js.map